import re
import time
from pprint import pprint
import requests
from logzero import logger


# 基于atxserver远程调用真机
class Device(object):
    def __init__(self, server_url, token):
        self.server_url = server_url
        self.token = token
        self.udid = None
        self.address = None

    def mark_url(self, path):
        if re.match(r"^https?://", path):
            return path
        return self.server_url + path

    def request_api(self, path, method="GET", **kwargs):
        kwargs['headers'] = {"Authorization": "Bearer " + self.token}
        r = requests.request(method, self.mark_url(path), **kwargs)
        try:
            r.raise_for_status()
        except requests.HTTPError:
            pprint(r.text)
        return r.json()

    def get_device(self, platform='android'):
        # 获取用户信息
        ret = self.request_api("/api/v1/user")
        logger.info(f"User: {ret['username']}")

        # 获取可用设备列表
        ret = self.request_api("/api/v1/devices", params={"usable": "true", "platform": platform})
        # logger.info(ret)
        if not ret['devices']:
            raise EnvironmentError("No devices")
        logger.info(f"Device count: {len(ret['devices'])}")

        # 占用设备
        device = ret['devices'][1]
        self.udid = device['udid']
        logger.info(f"Choose device: {device['properties']['name']} udid={self.udid}")
        ret = self.request_api("/api/v1/user/devices", method='post', json={"udid": self.udid})
        # logger.info(ret)

        # 获取占用设备信息
        ret = self.request_api("/api/v1/user/devices/" + self.udid)
        source = ret['device']['source']
        pprint(source)
        if platform == 'android':
            self.address = source['atxAgentAddress']
            # subprocess.run(['adb', 'connect', self.address])
            time.sleep(1)
        if platform == 'apple':
            self.address = source['wdaUrl']
        return self.address

    def release_device(self):
        # subprocess.run(['adb', 'disconnect', self.address])
        ret = self.request_api("/api/v1/user/devices/" + self.udid, method="delete")
        print(ret)






from .. import Provider as CompanyProvider


class Provider(CompanyProvider):
    formats = (
        "{{last_name}} {{company_suffix}}",
        "{{last_name}} {{last_name}} {{company_suffix}}",
        "{{last_name}}",
        "{{last_name}}",
    )

    nouns = (
        "a segurança",
        "o prazer",
        "o conforto",
        "a simplicidade",
        "a certeza",
        "a arte",
        "o poder",
        "o direito",
        "a possibilidade",
        "a vantagem",
        "a liberdade",
    )

    verbs = (
        "de conseguir",
        "de avançar",
        "de evoluir",
        "de mudar",
        "de inovar",
        "de ganhar",
        "de atingir os seus objetivos",
        "de concretizar seus projetos",
        "de realizar seus sonhos",
    )

    attributes = (
        "de maneira eficaz",
        "mais rapidamente",
        "mais facilmente",
        "simplesmente",
        "com toda a tranquilidade",
        "antes de tudo",
        "naturalmente",
        "sem preocupação",
        "em estado puro",
        "com força total",
        "direto da fonte",
        "com confiança",
    )

    company_suffixes = ("S/A", "S.A.", "Lda.", "e Filhos")



from .. import Provider as InternetProvider


class Provider(InternetProvider):

    free_email_domains = (
        "aol.de",
        "gmail.com",
        "gmx.de",
        "googlemail.com",
        "hotmail.de",
        "web.de",
        "yahoo.de",
    )
    tlds = ("com", "com", "com", "net", "org", "de", "de", "de")

    replacements = (
        ("ä", "ae"),
        ("Ä", "Ae"),
        ("ö", "oe"),
        ("Ö", "Oe"),
        ("ü", "ue"),
        ("Ü", "Ue"),
        ("é", "e"),
        ("É", "E"),
        ("à", "a"),
        ("À", "A"),
        ("ß", "ss"),
    )



import json
import os
import sys
import time
import allure
from baseImage import Image, Rect
from selenium import webdriver
from qrunner.utils.config import conf
from qrunner.utils.log import logger
from qrunner.utils.webdriver_manager_extend import ChromeDriverManager
from qrunner.core.android.driver import AndroidDriver


# class H5Config:
#     webview_version = '77.0.3865.40'


# # 重启chromedriver的装饰器
# def relaunch(func):
#     def wrapper(self, *args, **kwargs):
#         try:
#             return func(self, *args, **kwargs)
#         except requests.exceptions.ConnectionError as _:
#             logger.warning("h5 driver error, relaunch now.")
#             self.d = H5Driver(conf.get_name('app', 'serial_no'))
#     return wrapper


class H5Driver(object):
    # _instance = {}
    #
    # def __new__(cls, serial_no=None):
    #     if not serial_no:
    #         serial_no = conf.get_name('app', 'serial_no')
    #     if serial_no not in cls._instance:
    #         cls._instance[serial_no] = super().__new__(cls)
    #     return cls._instance[serial_no]

    def __init__(self, android_driver: AndroidDriver):
        serial_no = android_driver.serial_no
        pkg_name = android_driver.pkg_name

        # # 判断应用是否已启动
        # pid = android_driver.wait_app_running()
        # if pid:
        #     logger.debug('应用已启动')
        # else:
        #     logger.debug('应用未启动，h5Driver初始化失败')
        #     sys.exit()

        logger.info(f'启动H5Driver for {serial_no}')
        options = webdriver.ChromeOptions()
        options.add_experimental_option('androidDeviceSerial', serial_no)
        options.add_experimental_option('androidPackage', pkg_name)
        options.add_experimental_option('androidUseRunningApp', True)
        options.add_experimental_option('androidProcess', pkg_name)
        version_list = conf.get_item('app', 'webview_version')
        for version in version_list:
            try:
                path_ = ChromeDriverManager(version=version).install()
                self.d = webdriver.Chrome(
                    executable_path=path_,
                    options=options
                )
                logger.debug('h5Driver初始化成功')
                logger.debug(f'版本 {version} 配对成功')
                break
            except:
                logger.debug(f'版本 {version} 配对失败')
                continue
        else:
            logger.debug(f'配置的版本号都不对: {version_list}')
            sys.exit()

        # 设置页面加载超时时间
        try:
            timeout = int(conf.get_item('common', 'timeout'))
        except:
            pass
        else:
            self.d.set_page_load_timeout(timeout)

    # @staticmethod
    # def get_instance(cls, serial_no=None):
    #     if serial_no not in cls._instance:
    #         logger.info(f'{serial_no} Create h5 driver singleton')
    #         return H5Driver(serial_no)
    #     return H5Driver._instance[serial_no]

    def open_url(self, url):
        logger.info(f'访问: {url}')
        self.d.get(url)

    def back(self):
        logger.info('返回上一页')
        self.d.back()

    def screenshot(self, file_name, rect=None):
        """
        截图并保存到预定路径并上传到allure报告
        @param file_name: foo.png or fool
        @param rect: 列表
        @return:
        """
        # 把文件名处理成test.png的样式
        if '.' in file_name:
            file_name = file_name.split(r'.')[0]
        # 截图并保存到当前目录的images文件夹中
        img_dir = os.path.join(os.getcwd(), 'images')
        if os.path.exists(img_dir) is False:
            os.mkdir(img_dir)
        time_str = time.strftime('%Y年%m月%d日 %H时%M分%S秒')
        file_path = os.path.join(img_dir,
                                 f'{time_str}-{file_name}.png')
        logger.info(f'截图保存至: {file_path}')
        self.d.save_screenshot(file_path)

        # 标记
        if rect is not None:
            img = Image(file_path)
            img.rectangle(Rect(*rect), color=(0, 0, 255), thickness=3)
            img.imwrite(file_path)

        # 截图并上传到allure报告
        logger.info(f'截图上传至allure报告')
        allure.attach.file(file_path,
                           attachment_type=allure.attachment_type.PNG,
                           name=f'{time_str}-{file_name}.png')

    # @relaunch
    # def upload_pic(self, filename):
    #     # allure.attach.file(self.d.get_screenshot_as_png(),
    #     #                    attachment_type=allure.attachment_type.PNG,
    #     #                    name=f'{filename}')
    #     self.d.save_screenshot('tmp.png')
    #     allure.attach.file('tmp.png', attachment_type=allure.attachment_type.PNG,
    #                        name=f'{filename}-{get_time()}')
    #     os.remove('tmp.png')

    @property
    def page_content(self):
        page_source = self.d.page_source
        logger.info(f'获取页面内容: \n{page_source}')
        return page_source

    def max_window(self):
        logger.info('窗口设置全屏')
        self.d.maximize_window()

    def set_window_size(self, width, height):
        logger.info('设置窗口长宽')
        self.d.set_window_size(width, height)

    def get_windows(self):
        logger.info(f'获取当前打开的窗口列表')
        return self.d.window_handles

    def switch_window(self, old_windows):
        logger.info('切换到最新的window')
        current_windows = self.d.window_handles
        newest_window = [window for window in current_windows if window not in old_windows][0]
        self.d.switch_to.window(newest_window)

    def window_scroll(self, width, height):
        logger.info(f'设置scroll bar的宽高')
        js = "window.scrollTo({w},{h});".format(w=str(width), h=str(height))
        self.d.execute_script(js)

    def switch_to_frame(self, frame_id):
        logger.info(f'切换到frame {frame_id}')
        self.d.switch_to.frame(frame_id)

    def switch_to_frame_out(self):
        logger.info('从frame中切出来')
        self.d.switch_to.default_content()

    def execute_js(self, script, *args):
        logger.info(f'执行js脚本: \n{script}')
        self.d.execute_script(script, *args)

    def click(self, element):
        logger.info(f'点击元素: {element}')
        self.d.execute_script('arguments[0].click();', element)

    def quit(self):
        logger.info('退出浏览器')
        self.d.quit()

    def close(self):
        logger.info('关闭当前页签')
        self.d.close()

    def add_cookies(self, cookies: list):
        for cookie in cookies:
            self.d.add_cookie(cookie)

    def get_cookies(self):
        logger.info('获取cookies')
        cookies = self.d.get_cookies()
        logger.info(cookies)
        return cookies

    def get_cookie(self, name):
        logger.info(f'获取cookie: {name}')
        cookie = self.d.get_cookie(name)
        logger.info(cookie)
        return cookie

    def delete_all_cookies(self):
        logger.info('删除所有cookie')
        self.d.delete_all_cookies()

    def delete_cookie(self, name):
        logger.info(f'删除cookie: {name}')
        self.d.delete_cookie(name)

    def refresh(self):
        logger.info(f'刷新当前页')
        self.d.refresh()

    def get_title(self):
        logger.info('获取页面标题')
        title = self.d.title
        logger.info(title)
        return title

    def get_url(self):
        logger.info('获取页面url')
        url = self.d.current_url
        logger.info(url)
        return url

    def get_alert_text(self):
        logger.info('获取alert的文本')
        alert_text = self.d.switch_to.alert.text
        logger.info(alert_text)
        return alert_text

    def accept_alert(self):
        logger.info('同意确认框')
        self.d.switch_to.alert.accept()

    def dismiss_alert(self):
        logger.info('取消确认框')
        self.d.switch_to.alert.dismiss()








import inspect
import warnings

from typing import Any, Dict, List, Optional, Tuple, Union

from qrunner.utils.faker.generator import Generator
from qrunner.utils.faker.providers import BaseProvider
from qrunner.utils.faker.proxy import Faker


class Documentor:
    def __init__(self, generator: Union[Generator, Faker]) -> None:
        """
        :param generator: a localized Generator with providers filled,
                          for which to write the documentation
        :type generator: faker.Generator()
        """
        self.generator = generator
        self.max_name_len: int = 0
        self.already_generated: List[str] = []

    def get_formatters(
        self,
        locale: Optional[str] = None,
        excludes: Optional[List[str]] = None,
        **kwargs: Any,
    ) -> List[Tuple[BaseProvider, Dict[str, str]]]:
        self.max_name_len = 0
        self.already_generated = [] if excludes is None else excludes[:]
        formatters = []
        providers: List[BaseProvider] = self.generator.get_providers()
        for provider in providers[::-1]:  # reverse
            if locale and provider.__lang__ and provider.__lang__ != locale:
                continue
            formatters.append(
                (provider, self.get_provider_formatters(provider, **kwargs)),
            )
        return formatters

    def get_provider_formatters(
        self,
        provider: BaseProvider,
        prefix: str = "fake.",
        with_args: bool = True,
        with_defaults: bool = True,
    ) -> Dict[str, str]:
        formatters = {}

        for name, method in inspect.getmembers(provider, inspect.ismethod):
            # skip 'private' method and inherited methods
            if name.startswith("_") or name in self.already_generated:
                continue

            arguments = []
            faker_args: List[str] = []
            faker_kwargs = {}

            if name == "binary":
                faker_kwargs["length"] = 1024
            elif name in ["zip", "tar"]:
                faker_kwargs.update(
                    {
                        "uncompressed_size": 1024,
                        "min_file_size": 512,
                    }
                )

            if with_args:
                # retrieve all parameter
                argspec = inspect.getfullargspec(method)

                lst = [x for x in argspec.args if x not in ["self", "cls"]]
                for i, arg in enumerate(lst):

                    if argspec.defaults and with_defaults:

                        try:
                            default = argspec.defaults[i]
                            if isinstance(default, str):
                                default = repr(default)
                            else:
                                # TODO check default type
                                default = f"{default}"

                            arg = f"{arg}={default}"

                        except IndexError:
                            pass

                    arguments.append(arg)
                    if with_args == "first":
                        break

                if with_args != "first":
                    if argspec.varargs:
                        arguments.append("*" + argspec.varargs)
                    if argspec.varkw:
                        arguments.append("**" + argspec.varkw)

            # build fake method signature
            signature = f"{prefix}{name}({', '.join(arguments)})"

            try:
                # make a fake example
                example = self.generator.format(name, *faker_args, **faker_kwargs)
            except (AttributeError, ValueError) as e:
                warnings.warn(str(e))
                continue
            formatters[signature] = example

            self.max_name_len = max(self.max_name_len, len(signature))
            self.already_generated.append(name)

        return formatters

    @staticmethod
    def get_provider_name(provider_class: BaseProvider) -> str:
        return provider_class.__provider__



import logging
import sys


# 创建一个logger
logger = logging.getLogger()
logger.setLevel(logging.DEBUG)

# 再创建一个handler，用于输出到控制台
ch = logging.StreamHandler(sys.stdout)
ch.setLevel(logging.DEBUG)

# 定义handler的输出格式
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
ch.setFormatter(formatter)

# 给logger添加handler
if not logger.handlers:
    logger.addHandler(ch)




from ..en_PH import Provider as EnPhAutomotiveProvider


class Provider(EnPhAutomotiveProvider):
    """Implement automotive provider for ``fil_PH`` locale.

    There is no difference from the ``en_PH`` implementation.
    """

    pass



"""
This module is responsible for generating the check digit and formatting
ISBN numbers.
"""
from typing import Any, Optional


class ISBN:

    MAX_LENGTH = 13

    def __init__(
        self,
        ean: Optional[str] = None,
        group: Optional[str] = None,
        registrant: Optional[str] = None,
        publication: Optional[str] = None,
    ) -> None:
        self.ean = ean
        self.group = group
        self.registrant = registrant
        self.publication = publication


class ISBN13(ISBN):
    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        self.check_digit = self._check_digit()

    def _check_digit(self) -> str:
        """Calculate the check digit for ISBN-13.
        See https://en.wikipedia.org/wiki/International_Standard_Book_Number
        for calculation.
        """
        weights = (1 if x % 2 == 0 else 3 for x in range(12))
        body = "".join([part for part in [self.ean, self.group, self.registrant, self.publication] if part is not None])
        remainder = sum(int(b) * w for b, w in zip(body, weights)) % 10
        diff = 10 - remainder
        check_digit = 0 if diff == 10 else diff
        return str(check_digit)

    def format(self, separator: str = "") -> str:
        return separator.join(
            [
                part
                for part in [
                    self.ean,
                    self.group,
                    self.registrant,
                    self.publication,
                    self.check_digit,
                ]
                if part is not None
            ]
        )


class ISBN10(ISBN):
    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        self.check_digit = self._check_digit()

    def _check_digit(self) -> str:
        """Calculate the check digit for ISBN-10.
        See https://en.wikipedia.org/wiki/International_Standard_Book_Number
        for calculation.
        """
        weights = range(1, 10)
        body = "".join([part for part in [self.group, self.registrant, self.publication] if part is not None])
        remainder = sum(int(b) * w for b, w in zip(body, weights)) % 11
        check_digit = "X" if remainder == 10 else str(remainder)
        return str(check_digit)

    def format(self, separator: str = "") -> str:
        return separator.join(
            [
                part
                for part in [
                    self.group,
                    self.registrant,
                    self.publication,
                    self.check_digit,
                ]
                if part is not None
            ]
        )



from .. import BaseProvider, ElementsType

localized = True


class Provider(BaseProvider):
    ssn_formats: ElementsType = ("###-##-####",)

    def ssn(self) -> str:
        return self.bothify(self.random_element(self.ssn_formats))



from typing import Tuple

from .. import BaseProvider, ElementsType

localized = True


class Provider(BaseProvider):
    formats: ElementsType = (
        "{{last_name}} {{company_suffix}}",
        "{{last_name}}-{{last_name}}",
        "{{last_name}}, {{last_name}} and {{last_name}}",
    )

    company_suffixes: ElementsType = ("Inc", "and Sons", "LLC", "Group", "PLC", "Ltd")

    catch_phrase_words: Tuple[ElementsType, ...] = (
        (
            "Adaptive",
            "Advanced",
            "Ameliorated",
            "Assimilated",
            "Automated",
            "Balanced",
            "Business-focused",
            "Centralized",
            "Cloned",
            "Compatible",
            "Configurable",
            "Cross-group",
            "Cross-platform",
            "Customer-focused",
            "Customizable",
            "Decentralized",
            "De-engineered",
            "Devolved",
            "Digitized",
            "Distributed",
            "Diverse",
            "Down-sized",
            "Enhanced",
            "Enterprise-wide",
            "Ergonomic",
            "Exclusive",
            "Expanded",
            "Extended",
            "Face-to-face",
            "Focused",
            "Front-line",
            "Fully-configurable",
            "Function-based",
            "Fundamental",
            "Future-proofed",
            "Grass-roots",
            "Horizontal",
            "Implemented",
            "Innovative",
            "Integrated",
            "Intuitive",
            "Inverse",
            "Managed",
            "Mandatory",
            "Monitored",
            "Multi-channeled",
            "Multi-lateral",
            "Multi-layered",
            "Multi-tiered",
            "Networked",
            "Object-based",
            "Open-architected",
            "Open-source",
            "Operative",
            "Optimized",
            "Optional",
            "Organic",
            "Organized",
            "Persevering",
            "Persistent",
            "Phased",
            "Polarized",
            "Pre-emptive",
            "Proactive",
            "Profit-focused",
            "Profound",
            "Programmable",
            "Progressive",
            "Public-key",
            "Quality-focused",
            "Reactive",
            "Realigned",
            "Re-contextualized",
            "Re-engineered",
            "Reduced",
            "Reverse-engineered",
            "Right-sized",
            "Robust",
            "Seamless",
            "Secured",
            "Self-enabling",
            "Sharable",
            "Stand-alone",
            "Streamlined",
            "Switchable",
            "Synchronized",
            "Synergistic",
            "Synergized",
            "Team-oriented",
            "Total",
            "Triple-buffered",
            "Universal",
            "Up-sized",
            "Upgradable",
            "User-centric",
            "User-friendly",
            "Versatile",
            "Virtual",
            "Visionary",
            "Vision-oriented",
        ),
        (
            "24hour",
            "24/7",
            "3rdgeneration",
            "4thgeneration",
            "5thgeneration",
            "6thgeneration",
            "actuating",
            "analyzing",
            "asymmetric",
            "asynchronous",
            "attitude-oriented",
            "background",
            "bandwidth-monitored",
            "bi-directional",
            "bifurcated",
            "bottom-line",
            "clear-thinking",
            "client-driven",
            "client-server",
            "coherent",
            "cohesive",
            "composite",
            "context-sensitive",
            "contextually-based",
            "content-based",
            "dedicated",
            "demand-driven",
            "didactic",
            "directional",
            "discrete",
            "disintermediate",
            "dynamic",
            "eco-centric",
            "empowering",
            "encompassing",
            "even-keeled",
            "executive",
            "explicit",
            "exuding",
            "fault-tolerant",
            "foreground",
            "fresh-thinking",
            "full-range",
            "global",
            "grid-enabled",
            "heuristic",
            "high-level",
            "holistic",
            "homogeneous",
            "human-resource",
            "hybrid",
            "impactful",
            "incremental",
            "intangible",
            "interactive",
            "intermediate",
            "leadingedge",
            "local",
            "logistical",
            "maximized",
            "methodical",
            "mission-critical",
            "mobile",
            "modular",
            "motivating",
            "multimedia",
            "multi-state",
            "multi-tasking",
            "national",
            "needs-based",
            "neutral",
            "next generation",
            "non-volatile",
            "object-oriented",
            "optimal",
            "optimizing",
            "radical",
            "real-time",
            "reciprocal",
            "regional",
            "responsive",
            "scalable",
            "secondary",
            "solution-oriented",
            "stable",
            "static",
            "systematic",
            "systemic",
            "system-worthy",
            "tangible",
            "tertiary",
            "transitional",
            "uniform",
            "upward-trending",
            "user-facing",
            "value-added",
            "web-enabled",
            "well-modulated",
            "zero administration",
            "zero-defect",
            "zero tolerance",
        ),
        (
            "ability",
            "access",
            "adapter",
            "algorithm",
            "alliance",
            "analyzer",
            "application",
            "approach",
            "architecture",
            "archive",
            "artificial intelligence",
            "array",
            "attitude",
            "benchmark",
            "budgetary management",
            "capability",
            "capacity",
            "challenge",
            "circuit",
            "collaboration",
            "complexity",
            "concept",
            "conglomeration",
            "contingency",
            "core",
            "customer loyalty",
            "database",
            "data-warehouse",
            "definition",
            "emulation",
            "encoding",
            "encryption",
            "extranet",
            "firmware",
            "flexibility",
            "focus group",
            "forecast",
            "frame",
            "framework",
            "function",
            "functionalities",
            "Graphic Interface",
            "groupware",
            "Graphical User Interface",
            "hardware",
            "help-desk",
            "hierarchy",
            "hub",
            "implementation",
            "info-mediaries",
            "infrastructure",
            "initiative",
            "installation",
            "instruction set",
            "interface",
            "Internet solution",
            "intranet",
            "knowledge user",
            "knowledgebase",
            "Local Area Network",
            "leverage",
            "matrices",
            "matrix",
            "methodology",
            "middleware",
            "migration",
            "model",
            "moderator",
            "monitoring",
            "moratorium",
            "neural-net",
            "open architecture",
            "open system",
            "orchestration",
            "paradigm",
            "parallelism",
            "policy",
            "portal",
            "pricing structure",
            "process improvement",
            "product",
            "productivity",
            "project",
            "projection",
            "protocol",
            "secured line",
            "service-desk",
            "software",
            "solution",
            "standardization",
            "strategy",
            "structure",
            "success",
            "superstructure",
            "support",
            "synergy",
            "system engine",
            "task-force",
            "throughput",
            "time-frame",
            "toolset",
            "utilization",
            "website",
            "workforce",
        ),
    )

    bsWords: Tuple[ElementsType, ...] = (
        (
            "implement",
            "utilize",
            "integrate",
            "streamline",
            "optimize",
            "evolve",
            "transform",
            "embrace",
            "enable",
            "orchestrate",
            "leverage",
            "reinvent",
            "aggregate",
            "architect",
            "enhance",
            "incentivize",
            "morph",
            "empower",
            "envisioneer",
            "monetize",
            "harness",
            "facilitate",
            "seize",
            "disintermediate",
            "synergize",
            "strategize",
            "deploy",
            "brand",
            "grow",
            "target",
            "syndicate",
            "synthesize",
            "deliver",
            "mesh",
            "incubate",
            "engage",
            "maximize",
            "benchmark",
            "expedite",
            "re-intermediate",
            "whiteboard",
            "visualize",
            "repurpose",
            "innovate",
            "scale",
            "unleash",
            "drive",
            "extend",
            "engineer",
            "revolutionize",
            "generate",
            "exploit",
            "transition",
            "e-enable",
            "iterate",
            "cultivate",
            "matrix",
            "productize",
            "redefine",
            "re-contextualize",
        ),
        (
            "clicks-and-mortar",
            "value-added",
            "vertical",
            "proactive",
            "robust",
            "revolutionary",
            "scalable",
            "leading-edge",
            "innovative",
            "intuitive",
            "strategic",
            "e-business",
            "mission-critical",
            "sticky",
            "one-to-one",
            "24/7",
            "end-to-end",
            "global",
            "B2B",
            "B2C",
            "granular",
            "frictionless",
            "virtual",
            "viral",
            "dynamic",
            "24/365",
            "best-of-breed",
            "killer",
            "magnetic",
            "bleeding-edge",
            "web-enabled",
            "interactive",
            "dot-com",
            "back-end",
            "real-time",
            "efficient",
            "front-end",
            "distributed",
            "seamless",
            "extensible",
            "turn-key",
            "world-class",
            "open-source",
            "cross-platform",
            "cross-media",
            "synergistic",
            "bricks-and-clicks",
            "out-of-the-box",
            "enterprise",
            "integrated",
            "impactful",
            "wireless",
            "transparent",
            "next-generation",
            "cutting-edge",
            "user-centric",
            "visionary",
            "customized",
            "ubiquitous",
            "plug-and-play",
            "collaborative",
            "compelling",
            "holistic",
            "rich",
        ),
        (
            "synergies",
            "web-readiness",
            "paradigms",
            "markets",
            "partnerships",
            "infrastructures",
            "platforms",
            "initiatives",
            "channels",
            "eyeballs",
            "communities",
            "ROI",
            "solutions",
            "e-tailers",
            "e-services",
            "action-items",
            "portals",
            "niches",
            "technologies",
            "content",
            "vortals",
            "supply-chains",
            "convergence",
            "relationships",
            "architectures",
            "interfaces",
            "e-markets",
            "e-commerce",
            "systems",
            "bandwidth",
            "info-mediaries",
            "models",
            "mindshare",
            "deliverables",
            "users",
            "schemas",
            "networks",
            "applications",
            "metrics",
            "e-business",
            "functionalities",
            "experiences",
            "web services",
            "methodologies",
        ),
    )

    def company(self) -> str:
        """
        :example: 'Acme Ltd'
        """
        pattern: str = self.random_element(self.formats)
        return self.generator.parse(pattern)

    def company_suffix(self) -> str:
        """
        :example: 'Ltd'
        """
        return self.random_element(self.company_suffixes)

    def catch_phrase(self) -> str:
        """
        :example: 'Robust full-range hub'
        """
        return " ".join([self.random_element(word_list) for word_list in self.catch_phrase_words])

    def bs(self) -> str:
        """
        :example: 'integrate extensible convergence'
        """
        return " ".join([self.random_element(word_list) for word_list in self.bsWords])



from typing import Sequence, Tuple

from ... import BaseProvider


class Provider(BaseProvider):
    """
    Provider for Philippine mobile and landline telephone numbers

    This provider has methods that generate phone numbers specific to service providers whenever applicable, because the
    kinds of services, the quality of said services, and even the fees may vary depending on the service provider and
    the service location. This in turn, affects subscriber behavior, e.g. someone with a SIM from company X may be very
    unlikely to respond to calls and texts sent from a company Y SIM as the service charge might be more expensive. The
    provider methods are there to enable the creation of more "realistic" fake data for such cases.

    Additional Notes:
        - The Philippine telecommunication industry is dominated by the Globe-PLDT duopoly. Globe offers landline
          services under the Globe brand and mobile services under the Globe and TM brands. PLDT offers landline
          services under the PLDT brand, and its subsidiaries offer mobile services under the Smart, TNT, and SUN
          brands. The rest of the industry is shared by smaller players, and Bayantel is one of the more well-known
          players that provide landline services.
        - Globe mobile prefixes include both Globe and TM brands, and the Smart mobile prefixes include both Smart
          and TNT brands but not the SUN brand. Available sources only split the prefixes this way.
        - In October 2019, Area 2 landline numbers were migrated to an 8 digit scheme, while the rest of the
          country still uses the original 7 digit scheme. Area 2 is comprised of the whole National Capital
          Region (aka Metro Manila) and parts of surrounding provinces, and within this area, the service
          provider's identifier is included in every 8 digit landline number.

    Sources:
        - https://en.wikipedia.org/wiki/Telephone_numbers_in_the_Philippines
        - https://www.prefix.ph/prefixes/2019-updated-complete-list-of-philippine-mobile-network-prefixes/
        - https://powerpinoys.com/network-prefixes-philippines/
    """

    globe_mobile_number_prefixes: Tuple[str, ...] = (
        "817",
        "904",
        "905",
        "906",
        "915",
        "916",
        "917",
        "926",
        "927",
        "935",
        "936",
        "937",
        "945",
        "955",
        "956",
        "965",
        "966",
        "967",
        "973",
        "975",
        "976",
        "977",
        "978",
        "979",
        "994",
        "995",
        "996",
        "997",
    )
    smart_mobile_number_prefixes: Tuple[str, ...] = (
        "813",
        "907",
        "908",
        "909",
        "910",
        "911",
        "912",
        "913",
        "914",
        "918",
        "919",
        "920",
        "921",
        "928",
        "929",
        "930",
        "938",
        "939",
        "940",
        "946",
        "947",
        "948",
        "949",
        "950",
        "951",
        "961",
        "970",
        "981",
        "989",
        "992",
        "998",
        "999",
    )
    sun_mobile_number_prefixes: Tuple[str, ...] = (
        "922",
        "923",
        "924",
        "925",
        "931",
        "932",
        "933",
        "934",
        "941",
        "942",
        "943",
        "944",
    )
    globe_mobile_number_formats: Tuple[str, ...] = (
        "0{{globe_mobile_number_prefix}}-###-####",
        "+63{{globe_mobile_number_prefix}}-###-####",
    )
    smart_mobile_number_formats: Tuple[str, ...] = (
        "0{{smart_mobile_number_prefix}}-###-####",
        "+63{{smart_mobile_number_prefix}}-###-####",
    )
    sun_mobile_number_formats: Tuple[str, ...] = (
        "0{{sun_mobile_number_prefix}}-###-####",
        "+63{{sun_mobile_number_prefix}}-###-####",
    )
    mobile_number_formats: Tuple[str, ...] = (
        globe_mobile_number_formats + smart_mobile_number_formats + sun_mobile_number_formats
    )

    bayantel_landline_identifiers: Tuple[str, ...] = tuple(str(x) for x in range(3000, 3500))
    misc_landline_identifiers: Tuple[str, ...] = tuple(str(x) for x in range(5300, 5800)) + tuple(
        str(x) for x in range(6000, 6700)
    )
    non_area2_landline_area_codes: Tuple[str, ...] = (
        "32",
        "33",
        "34",
        "35",
        "36",
        "38",
        "42",
        "43",
        "44",
        "45",
        "46",
        "47",
        "48",
        "49",
        "52",
        "53",
        "54",
        "55",
        "56",
        "62",
        "63",
        "64",
        "65",
        "68",
        "72",
        "74",
        "75",
        "77",
        "78",
        "82",
        "83",
        "84",
        "85",
        "86",
        "87",
        "88",
    )
    globe_area2_landline_number_formats: Tuple[str, ...] = (
        "02-7###-####",
        "+632-7###-####",
    )
    pldt_area2_landline_number_formats: Tuple[str, ...] = (
        "02-8###-####",
        "+632-8###-####",
    )
    bayantel_area2_landline_number_formats: Tuple[str, ...] = (
        "02-{{bayantel_landline_identifier}}-####",
        "+632-{{bayantel_landline_identifier}}-####",
    )
    misc_area2_landline_number_formats: Tuple[str, ...] = (
        "02-{{misc_landline_identifier}}-####",
        "+632-{{misc_landline_identifier}}-####",
    )
    area2_landline_number_formats: Tuple[str, ...] = (
        globe_area2_landline_number_formats
        + pldt_area2_landline_number_formats
        + bayantel_area2_landline_number_formats
        + misc_area2_landline_number_formats
    )
    non_area2_landline_number_formats: Tuple[str, ...] = (
        "0{{non_area2_landline_area_code}}-###-####",
        "+63{{non_area2_landline_area_code}}-###-####",
    )
    landline_number_formats: Tuple[str, ...] = area2_landline_number_formats + non_area2_landline_number_formats

    def _create_phone_number(self, formats: Sequence[str]) -> str:
        pattern: str = self.random_element(formats)
        return self.numerify(self.generator.parse(pattern))

    def globe_mobile_number_prefix(self) -> str:
        return self.random_element(self.globe_mobile_number_prefixes)

    def smart_mobile_number_prefix(self) -> str:
        return self.random_element(self.smart_mobile_number_prefixes)

    def sun_mobile_number_prefix(self) -> str:
        return self.random_element(self.sun_mobile_number_prefixes)

    def bayantel_landline_identifier(self) -> str:
        return self.random_element(self.bayantel_landline_identifiers)

    def misc_landline_identifier(self) -> str:
        return self.random_element(self.misc_landline_identifiers)

    def non_area2_landline_area_code(self) -> str:
        return self.random_element(self.non_area2_landline_area_codes)

    def globe_mobile_number(self) -> str:
        return self._create_phone_number(self.globe_mobile_number_formats)

    def smart_mobile_number(self) -> str:
        return self._create_phone_number(self.smart_mobile_number_formats)

    def sun_mobile_number(self) -> str:
        return self._create_phone_number(self.sun_mobile_number_formats)

    def mobile_number(self) -> str:
        return self._create_phone_number(self.mobile_number_formats)

    def globe_area2_landline_number(self) -> str:
        return self._create_phone_number(self.globe_area2_landline_number_formats)

    def pldt_area2_landline_number(self) -> str:
        return self._create_phone_number(self.pldt_area2_landline_number_formats)

    def bayantel_area2_landline_number(self) -> str:
        return self._create_phone_number(self.bayantel_area2_landline_number_formats)

    def misc_area2_landline_number(self) -> str:
        return self._create_phone_number(self.misc_area2_landline_number_formats)

    def area2_landline_number(self) -> str:
        return self._create_phone_number(self.area2_landline_number_formats)

    def non_area2_landline_number(self) -> str:
        return self._create_phone_number(self.non_area2_landline_number_formats)

    def landline_number(self) -> str:
        return self._create_phone_number(self.landline_number_formats)



import smtplib
import time
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.header import Header


# 邮件通知
from qrunner.utils.allure_data import AllureData
from qrunner.utils.config import conf


class Mail:
    def __init__(self, host, user, password):
        self.host = host
        self.username = user
        self.password = password

    def send_mail(self, mail_data, receivers):
        print(f'向{receivers}发送邮件...')
        # 创建一个带附件的实例
        message = MIMEMultipart()
        message['From'] = Header(self.username)
        message['To'] = Header(",".join(receivers))
        message['Subject'] = Header(mail_data.get('title'), 'utf-8')

        # 邮件正文内容
        message.attach(MIMEText(mail_data.get('body'), 'plain', 'utf-8'))
        # 附件
        file_path = mail_data.get('file_path')
        if file_path:
            # 构造附件，传送当前目录下的文件
            att1 = MIMEText(open(file_path, 'r').read(), 'base64', 'utf-8')
            att1["Content-Type"] = 'application/octet-stream'
            # 这里的filename可以任意写，写什么名字，邮件中显示什么名字
            file_name = mail_data.get('file_name')
            att1["Content-Disposition"] = f'attachment; filename="{file_name}"'
            message.attach(att1)

        # 连接
        conn = smtplib.SMTP_SSL(self.host, 465)
        # 登录
        conn.login(self.username, self.password)
        # 发送邮件
        try:
            conn.sendmail(self.username, receivers, message.as_string())
        except Exception as e:
            print(f'发送失败: {str(e)}')
        else:
            print('发送成功')
        # 断开连接
        conn.quit()

    def send_report(self, title, report_url, receiver_list):
        report_path = conf.get_item('common', 'report_path')
        allure_data = AllureData(report_path).get_basic_info()
        total = allure_data.get('total')
        fail = allure_data.get('fail')
        passed = allure_data.get('passed')
        rate = allure_data.get('rate')

        # 邮件内容
        title = f'{title}({time.strftime("%m-%d %H:%M")})'
        body_str = '\n\n共 {0} 个用例，通过 {1} 个，失败 {2} 个，通过率 {3}%，详见: {4}'\
            .format(total, passed, fail, rate, report_url)
        msg_data = {
            'title': title,
            'body': body_str,
            'file_path': None,
            'file_name': None
        }

        # 发送邮件
        self.send_mail(msg_data, receiver_list)







import pkgutil
import sys

from importlib import import_module
from pathlib import Path
from types import ModuleType
from typing import List


def get_path(module: ModuleType) -> str:
    if getattr(sys, "frozen", False):
        # frozen

        if getattr(sys, "_MEIPASS", False):
            # PyInstaller
            lib_dir = Path(getattr(sys, "_MEIPASS"))
        else:
            # others
            lib_dir = Path(sys.executable).parent / "lib"

        path = lib_dir.joinpath(*module.__package__.split("."))  # type: ignore
    else:
        # unfrozen
        if module.__file__ is not None:
            path = Path(module.__file__).parent
        else:
            raise RuntimeError(f"Can't find path from module `{module}.")
    return str(path)


def list_module(module: ModuleType) -> List[str]:
    path = get_path(module)

    if getattr(sys, "_MEIPASS", False):
        # PyInstaller
        return [file.parent.name for file in Path(path).glob("*/__init__.py")]
    else:
        return [name for _, name, is_pkg in pkgutil.iter_modules([str(path)]) if is_pkg]


def find_available_locales(providers: List[str]) -> List[str]:
    available_locales = set()

    for provider_path in providers:

        provider_module = import_module(provider_path)
        if getattr(provider_module, "localized", False):
            langs = list_module(provider_module)
            available_locales.update(langs)
    return sorted(available_locales)


def find_available_providers(modules: List[ModuleType]) -> List[str]:
    available_providers = set()
    for providers_mod in modules:
        if providers_mod.__package__:
            providers = [
                ".".join([providers_mod.__package__, mod]) for mod in list_module(providers_mod) if mod != "__pycache__"
            ]
            available_providers.update(providers)
    return sorted(available_providers)



from .. import Provider as DateTimeProvider


class Provider(DateTimeProvider):

    DAY_NAMES = {
        "0": "Κυριακή",
        "1": "Δευτέρα",
        "2": "Τρίτη",
        "3": "Τετάρτη",
        "4": "Πέμπτη",
        "5": "Παρασκευή",
        "6": "Σάββατο",
    }

    MONTH_NAMES = {
        "01": "Ιανουάριος",
        "02": "Φεβρουάριος",
        "03": "Μάρτιος",
        "04": "Απρίλιος",
        "05": "Μάιος",
        "06": "Ιούνιος",
        "07": "Ιούλιος",
        "08": "Αύγουστος",
        "09": "Σεπτέμβριος",
        "10": "Οκτώβριος",
        "11": "Νοέμβριος",
        "12": "Δεκέμβριος",
    }

    def day_of_week(self):
        day = self.date("%w")
        return self.DAY_NAMES[day]

    def month_name(self):
        month = self.month()
        return self.MONTH_NAMES[month]



from .. import Provider as LoremProvider


class Provider(LoremProvider):
    """Implement lorem provider for ``de_DE`` locale.

    Word list is based on the source below, and some words have been removed
    because of some duplications.

    Sources:

    - https://www.gut1.de/grundwortschatz/grundwortschatz-500/
    """

    word_list = (
        "ab",
        "Abend",
        "aber",
        "acht",
        "Affe",
        "alle",
        "allein",
        "als",
        "also",
        "alt",
        "am",
        "an",
        "andere",
        "anfangen",
        "Angst",
        "antworten",
        "Apfel",
        "Arbeit",
        "arbeiten",
        "Arzt",
        "auch",
        "auf",
        "Auge",
        "aus",
        "Auto",
        "baden",
        "bald",
        "Ball",
        "bauen",
        "Bauer",
        "Baum",
        "bei",
        "beide",
        "beim",
        "Bein",
        "Beispiel",
        "beißen",
        "bekommen",
        "Berg",
        "besser",
        "Bett",
        "Bild",
        "bin",
        "bis",
        "blau",
        "bleiben",
        "Blume",
        "Boden",
        "brauchen",
        "braun",
        "Brief",
        "bringen",
        "Brot",
        "Bruder",
        "Buch",
        "böse",
        "da",
        "dabei",
        "dafür",
        "damit",
        "danach",
        "dann",
        "daran",
        "darauf",
        "darin",
        "das",
        "dauern",
        "davon",
        "dazu",
        "dein",
        "dem",
        "den",
        "denken",
        "denn",
        "der",
        "deshalb",
        "dich",
        "dick",
        "die",
        "Ding",
        "dir",
        "doch",
        "Dorf",
        "dort",
        "draußen",
        "drehen",
        "drei",
        "dumm",
        "dunkel",
        "durch",
        "dürfen",
        "eigentlich",
        "ein",
        "einfach",
        "einige",
        "einigen",
        "einmal",
        "Eis",
        "Eltern",
        "Ende",
        "endlich",
        "er",
        "Erde",
        "erklären",
        "erschrecken",
        "erst",
        "erzählen",
        "es",
        "essen",
        "Essen",
        "etwas",
        "fahren",
        "Fahrrad",
        "fallen",
        "Familie",
        "fangen",
        "fast",
        "fehlen",
        "Fenster",
        "Ferien",
        "fertig",
        "fest",
        "Feuer",
        "fiel",
        "finden",
        "Finger",
        "Fisch",
        "Flasche",
        "fliegen",
        "Frage",
        "fragen",
        "Frau",
        "frei",
        "fressen",
        "Freude",
        "freuen",
        "Freund",
        "fröhlich",
        "früh",
        "früher",
        "Fuß",
        "Fußball",
        "fährt",
        "führen",
        "fünf",
        "für",
        "gab",
        "ganz",
        "gar",
        "Garten",
        "geben",
        "Geburtstag",
        "gefährlich",
        "gegen",
        "gehen",
        "gehören",
        "gelb",
        "Geld",
        "genau",
        "gerade",
        "gern",
        "Geschenk",
        "Geschichte",
        "Gesicht",
        "gestern",
        "gesund",
        "gewinnen",
        "gibt",
        "ging",
        "Glas",
        "glauben",
        "gleich",
        "Glück",
        "glücklich",
        "Gott",
        "groß",
        "grün",
        "gut",
        "Haare",
        "haben",
        "halbe",
        "halten",
        "Hand",
        "hart",
        "Hase",
        "hat",
        "Haus",
        "heiß",
        "heißen",
        "helfen",
        "her",
        "heraus",
        "Herr",
        "Herz",
        "heute",
        "hier",
        "Hilfe",
        "Himmel",
        "hin",
        "hinein",
        "hinter",
        "hoch",
        "holen",
        "Hund",
        "Hunger",
        "hängen",
        "hören",
        "ich",
        "ihm",
        "ihn",
        "ihr",
        "im",
        "immer",
        "in",
        "ins",
        "ist",
        "ja",
        "Jahr",
        "jeder",
        "jetzt",
        "jung",
        "Junge",
        "kalt",
        "kam",
        "kann",
        "Katze",
        "kaufen",
        "kein",
        "kennen",
        "Kind",
        "Klasse",
        "klein",
        "klettern",
        "kochen",
        "kommen",
        "Kopf",
        "krank",
        "kurz",
        "können",
        "Küche",
        "lachen",
        "Land",
        "lange",
        "langsam",
        "las",
        "lassen",
        "laufen",
        "laut",
        "leben",
        "Leben",
        "legen",
        "Lehrer",
        "Lehrerin",
        "leicht",
        "leise",
        "lernen",
        "lesen",
        "letzte",
        "Leute",
        "Licht",
        "lieb",
        "liegen",
        "ließ",
        "Loch",
        "los",
        "Luft",
        "lustig",
        "machen",
        "mal",
        "Mama",
        "man",
        "Mann",
        "Maus",
        "Meer",
        "mehr",
        "mein",
        "Mensch",
        "merken",
        "mich",
        "Milch",
        "Minute",
        "Minutenmir",
        "mit",
        "Monat",
        "Monate",
        "Musik",
        "Mutter",
        "Mädchen",
        "mögen",
        "möglich",
        "müde",
        "müssen",
        "nach",
        "Nacht",
        "nah",
        "Name",
        "Nase",
        "nass",
        "natürlich",
        "neben",
        "nehmen",
        "nein",
        "nennen",
        "neu",
        "neun",
        "nicht",
        "nichts",
        "nie",
        "nimmt",
        "noch",
        "nun",
        "nur",
        "nächste",
        "nämlich",
        "ob",
        "oben",
        "oder",
        "offen",
        "oft",
        "ohne",
        "Oma",
        "Onkel",
        "Opa",
        "packen",
        "Papa",
        "Pferd",
        "Platz",
        "plötzlich",
        "Polizei",
        "Rad",
        "rechnen",
        "reich",
        "reiten",
        "rennen",
        "richtig",
        "rot",
        "rufen",
        "ruhig",
        "rund",
        "Sache",
        "sagen",
        "schaffen",
        "schauen",
        "scheinen",
        "schenken",
        "schicken",
        "Schiff",
        "schlafen",
        "schlagen",
        "schlecht",
        "schlimm",
        "Schluss",
        "Schnee",
        "schnell",
        "schon",
        "schreiben",
        "schreien",
        "Schuh",
        "Schule",
        "schwarz",
        "schwer",
        "Schwester",
        "schwimmen",
        "schön",
        "Schüler",
        "sechs",
        "See",
        "sehen",
        "sehr",
        "sein",
        "seit",
        "Seite",
        "selbst",
        "setzen",
        "sich",
        "sicher",
        "sie",
        "sieben",
        "sieht",
        "sind",
        "singen",
        "sitzen",
        "so",
        "sofort",
        "Sohn",
        "sollen",
        "Sommer",
        "Sonne",
        "Sonntag",
        "sonst",
        "Spaß",
        "Spiel",
        "spielen",
        "sprechen",
        "springen",
        "spät",
        "später",
        "Stadt",
        "stark",
        "stehen",
        "steigen",
        "Stein",
        "Stelle",
        "stellen",
        "Straße",
        "Stunde",
        "Stück",
        "suchen",
        "Tag",
        "Tante",
        "Teller",
        "tief",
        "Tier",
        "Tisch",
        "tot",
        "tragen",
        "traurig",
        "treffen",
        "trinken",
        "tun",
        "turnen",
        "Tür",
        "Uhr",
        "um",
        "und",
        "uns",
        "unser",
        "unten",
        "unter",
        "Vater",
        "vergessen",
        "verkaufen",
        "verlieren",
        "verstecken",
        "verstehen",
        "versuchen",
        "viel",
        "vielleicht",
        "vier",
        "Vogel",
        "voll",
        "vom",
        "von",
        "vor",
        "vorbei",
        "Wagen",
        "wahr",
        "Wald",
        "war",
        "warm",
        "warten",
        "warum",
        "was",
        "waschen",
        "Wasser",
        "weg",
        "Weg",
        "Weihnachten",
        "weil",
        "weinen",
        "weit",
        "weiter",
        "weiß",
        "Welt",
        "wenig",
        "wenn",
        "wer",
        "werden",
        "werfen",
        "Wetter",
        "wichtig",
        "wie",
        "wieder",
        "Wiese",
        "will",
        "Winter",
        "wir",
        "wird",
        "wirklich",
        "wissen",
        "Wissen",
        "wo",
        "Woche",
        "wohl",
        "wohnen",
        "Wohnung",
        "wollen",
        "Wort",
        "wünschen",
        "Zahl",
        "zehn",
        "zeigen",
        "Zeit",
        "Zeitung",
        "ziehen",
        "Zimmer",
        "zu",
        "Zug",
        "zum",
        "zur",
        "zurück",
        "zusammen",
        "zwei",
        "zwischen",
        "öffnen",
        "über",
        "überall",
    )



from .. import Provider as AddressProvider


class Provider(AddressProvider):

    # Building numbers don't go higher than a 1000
    building_number_formats = ("%##", "%#", "%")

    street_name_formats = ("{{dk_street_name}}{{street_suffix}}",)

    street_address_formats = ("{{street_name}} {{building_number}}",)

    street_names = (
        "Aberdeen",
        "Dompap",
        "Abildgaards",
        "Skovhare",
        "Svalehale",
        "Abrikos",
        "Absalons",
        "Adel",
        "Admiral",
        "Adrians",
        "Agerlands",
        "Aggersborg",
        "Aggersvold",
        "Agger",
        "Agnete",
        "Ahlefeldts",
        "Ahlmanns",
        "Ahorns",
        "Ahrenkildes",
        "Albaniens",
        "Aldersro",
        "Allers",
        "Alexandria",
        "Alliance",
        "Alperose",
        "Als",
        "Alsted",
        "Amagerbro",
        "Amagerfælled",
        "Amager",
        "Amagermotoren",
        "Amager Strand",
        "Amalie",
        "Amalie Skrams",
        "Amerika",
        "Amsterdam",
        "Angel",
        "Anneberg",
        "Anneke",
        "Annex",
        "Ansgars",
        "Antoinette",
        "Antoni",
        "Apollo",
        "Arabiens",
        "Arendals",
        "Arkona",
        "Armeniens",
        "Arne Jacobsens",
        "Arnes",
        "Arresø",
        "Arsenal",
        "Artilleri",
        "Asger Jorns",
        "Askø",
        "Asminderød",
        "Asnæs",
        "Assens",
        "Asters",
        "Astrup",
        "Asyl",
        "Athens",
        "Augusta",
        "Australiens",
        "Backers",
        "Badens",
        "Badstue",
        "Bager",
        "Baggesens",
        "Bakke",
        "Balders",
        "Ballum",
        "Baltika",
        "Bandholm",
        "Bangerts",
        "Bangsbo",
        "Bardenfleths",
        "Søfly",
        "Bartholins",
        "Basnæs",
        "Bastion",
        "Bavnager",
        "Bavnehøj",
        "Beate",
        "Bechgaards",
        "Bedford",
        "Beethovens",
        "Beldringe",
        "Belfast",
        "Belgiens",
        "Bellahøj",
        "Belle de Boskoop",
        "Bellida",
        "Bellis",
        "Bellmans",
        "Bergens",
        "Berggreens",
        "Bergthoras",
        "Bernstorffs",
        "Bevtoft",
        "Biens",
        "Billed",
        "Billesborg",
        "Birkager",
        "Birkedommer",
        "Birke",
        "Birkholm",
        "Birma",
        "Bisidder",
        "Bispebjerg",
        "Bispe",
        "Bissens",
        "Bjelkes",
        "Bjergmarks",
        "Bjergsted",
        "Bjernede",
        "Bjerregårds Side",
        "Bjerregårds",
        "Bjørneklo",
        "Bjørnsons",
        "Blanka",
        "Blegdams",
        "Blekinge",
        "Blushøj",
        "Blytækker",
        "Blåbær",
        "Blågårds",
        "Blåmejse",
        "Blåmunke",
        "Bodils",
        "Boeslunde",
        "Bogense",
        "Bogenæs",
        "Bogfinke",
        "Bogholder",
        "Bogtrykker",
        "Bohlendach",
        "Bolands",
        "Boldhus",
        "Bolton",
        "Bomhus",
        "Bomslutter",
        "Bomulds",
        "Bordeaux",
        "Borgbjergs",
        "Borger",
        "Borgmester Jensens",
        "Borgskriver",
        "Borneo",
        "Bornholms",
        "Borreby",
        "Borthigs",
        "Borups",
        "Boserup",
        "Botofte",
        "Boyes",
        "Brages",
        "Bramminge",
        "Bramslykke",
        "Bratskov",
        "Bredahls",
        "Bredelands",
        "Bred",
        "Bregnerød",
        "Breidablik",
        "Bremens",
        "Breslau",
        "Brigården",
        "Bri",
        "Bristol",
        "Broager",
        "Brobergs",
        "Brofoged",
        "Brohus",
        "Broksø",
        "Brolægger",
        "Brombær",
        "Brorsons",
        "Brydes",
        "Brygger",
        "Bryggeri",
        "Brynhilde",
        "Bryssel",
        "Brøndkær",
        "Brøndum",
        "Brøndæble",
        "Brønshøjgård",
        "Brønshøjholms",
        "Brønshøj Kirke",
        "Brønshøj",
        "Bulbjerg",
        "Bulgariens",
        "Buntmager",
        "Burmeisters",
        "Bustrup",
        "Byager",
        "Bygholm",
        "Byglands",
        "Bygmester",
        "Bygård",
        "Bykilde",
        "Bymose",
        "Bækkeskov",
        "Bøhmens",
        "Bøllegård",
        "Bøllemosegårds",
        "Børglum",
        "Børs",
        "Børskov",
        "Bådehavns",
        "Bådsmands",
        "Calais",
        "Capri",
        "Carl Nielsens",
        "Carls",
        "Carstens",
        "Castbergs",
        "Ceylon",
        "Christen Bergs",
        "Christian II's",
        "Christiansborg",
        "Christianshavns Vold",
        "Christiansminde",
        "Classens",
        "Clausholm",
        "Clematis",
        "Colbjørnsens",
        "Collins",
        "Container",
        "Cox Orange",
        "Cumberlands",
        "Cyperns",
        "Cæcilia",
        "Dag Hammarskjölds",
        "Dagmars",
        "Dagø",
        "Dahlerups",
        "Dalby",
        "Dalmose",
        "Dalslands",
        "Damager",
        "Dampfærge",
        "Dannebrogs",
        "Danneskiold-Samsøes",
        "Dannevirke",
        "Danshøj",
        "Danstrup",
        "Degnemose",
        "Degneæble",
        "Delfin",
        "Delos",
        "Derby",
        "Dige",
        "Dirchs",
        "Donau",
        "Dorthea",
        "Dovre",
        "Dragsholm",
        "Drechsels",
        "Drejer",
        "Drejø",
        "Drogdens",
        "Dronning Dagmars",
        "Dronning Elisabeths",
        "Dronningens",
        "Dronningens Tvær",
        "Dronninglund",
        "Dublin",
        "Dunhammer",
        "Dunkerque",
        "Dybbøls",
        "Dybendals",
        "Dybens",
        "Dyvekes",
        "Dønnerup",
        "Ebbe Rodes",
        "Eberts",
        "Eckersbergs",
        "Edel Sauntes",
        "Edelsminde",
        "Efterslægts",
        "Egebæks",
        "Ege",
        "Egelykke",
        "Egemarke",
        "Egholm",
        "Egils",
        "Ehlers",
        "Ejdersted",
        "Ejler Billes",
        "Ekvipagemester",
        "Elba",
        "Elias",
        "Ellebjerg",
        "Elme",
        "Elmelunde",
        "Elsdyrs",
        "Elselille",
        "Elstar",
        "Elværks",
        "Emblas",
        "Emdrup Eng",
        "Emdrupgårds",
        "Emdrup Kær",
        "Emdrup Mose",
        "Emdrup",
        "Enebær",
        "Engblomme",
        "Engdals",
        "Engelholm",
        "Engelsteds",
        "Enghave",
        "Englands",
        "Engskifte",
        "Eng",
        "Enigheds",
        "Enveloppe",
        "Erantis",
        "Eriks",
        "Eriksholm",
        "Eschrichts",
        "Eskadre",
        "Eskilds",
        "Eskildstrup",
        "Eskjær",
        "Esrom",
        "Esthers",
        "Estlands",
        "Eton",
        "Ewalds",
        "Fabrikmester",
        "Fafners",
        "Fajance",
        "Fakse",
        "Fakse Tvær",
        "Faksinge",
        "Falke",
        "Fanø",
        "Farum",
        "Farver",
        "Fehmerns",
        "Femkløver",
        "Fengers",
        "Fenris",
        "Fensmark",
        "Ferring",
        "Fersken",
        "Finlands",
        "Fiol",
        "Firkløver",
        "Fiskedams",
        "Fjenneslev",
        "Fladstjerne",
        "Flaske",
        "Flensborg",
        "Flidsager",
        "Flinterup",
        "Floras",
        "Florens",
        "Florida",
        "Flors",
        "Folevads",
        "Folke Bernadottes",
        "Forbindelses",
        "Fordresgård",
        "Formosa",
        "Fortun",
        "Fossgårds",
        "Fragt",
        "Frankrigs",
        "Fredensborg",
        "Fredens",
        "Fredenshøj",
        "Fredericia",
        "Frederiksberg",
        "Frederiksborg",
        "Frederiks",
        "Frederiksgårds",
        "Frederiksstads",
        "Frederikssunds",
        "Luftmarine",
        "Frejas",
        "Fremads",
        "Freunds",
        "Frilands",
        "Frimester",
        "Fruebjerg",
        "Fuglager",
        "Fuglefænger",
        "Fuglsang",
        "Funkia",
        "Fussings",
        "Fyens",
        "Fyrbøder",
        "Fyrtårn",
        "Fælled",
        "Fælledens Passage",
        "Færgehavns",
        "Følfod",
        "Førslev",
        "Fåborg",
        "Gadekær",
        "Gadstrup",
        "Galions",
        "Gamle Vasby",
        "Gammel Jernbane",
        "Gammel Konge",
        "Gammel Køge Lande",
        "Gammeltofts",
        "Gartner",
        "Gartneri",
        "Gasværks",
        "Gaunø",
        "Gavlhus",
        "Gearhals",
        "Gefions",
        "Geislers",
        "Genua",
        "Georgine",
        "Gerbrands",
        "Gerdas",
        "Gerlev",
        "Gerners",
        "Gerts",
        "Geysers",
        "Gilbjerg",
        "Gimles",
        "Gislinge",
        "Gitter",
        "Gjorslev",
        "Gladbo",
        "Gladiolus",
        "Glas",
        "Glente",
        "Glinkas",
        "Glommens",
        "Glucks",
        "Glumsø",
        "Glückstadts",
        "Glænø",
        "Godsbane",
        "Godthåbs",
        "Gorms",
        "Gothers",
        "Gransanger",
        "Greis",
        "Grenå",
        "Grevinge",
        "Gribskov",
        "Griffenfelds",
        "Grysgårds",
        "Grækenlands",
        "Grønager",
        "Grøndals Park",
        "Grøndalsvænge",
        "Grønjords",
        "Grønløkke",
        "Grønne",
        "Grønnehave",
        "Grønnemose",
        "Grønris",
        "Gråbrødre",
        "Gråbynke",
        "Gråspurve",
        "Gråstens",
        "Gudenå",
        "Guldager",
        "Guldbergs",
        "Guldstjerne",
        "Gulkløver",
        "Gullands",
        "Gullfoss",
        "Gunhilds",
        "Gunløgs",
        "Gyldenlak",
        "Gyldenløves",
        "Gyldenris",
        "Gyrite",
        "Gyrstinge",
        "Gyvel",
        "Gørtler",
        "Gårdfæste",
        "Gårdtofte",
        "Gåsebæks",
        "Gåse",
        "Gåseurt",
        "Haderslev",
        "Hagbard",
        "Hagested",
        "Haifa",
        "Haldager",
        "Halfdans",
        "Halgreens",
        "Hallands",
        "Hallins",
        "Halsskov",
        "Hambros",
        "Hamlets",
        "Hammelstrup",
        "Hammerens",
        "Hammerichs",
        "Hammershus",
        "Hannemanns",
        "Hannover",
        "Hans Bogbinders",
        "Hanssted",
        "Hanstholm",
        "Haralds",
        "Haraldsted",
        "Harboøre",
        "Hardanger",
        "Hardenberg",
        "Hare",
        "Hareskov",
        "Harrestrup",
        "J.P.E. Hartmanns",
        "Harwich",
        "Hassel",
        "Hastings",
        "Hauser",
        "Havdrup",
        "Havkajak",
        "Havne",
        "Havre",
        "Havsgårds",
        "Haydns",
        "Hedeby",
        "Hedegaards",
        "Hedemanns",
        "Heibergs",
        "Heils",
        "Heimdals",
        "Heines",
        "Heises",
        "Hejre",
        "Heklas",
        "Heldbo",
        "Helgesens",
        "Helgolands",
        "Helikons",
        "Hellas",
        "Hellebæk",
        "Helleliden",
        "Hellested",
        "Helsingborg",
        "Helsingørmotoren",
        "Hemsedals",
        "Hendon",
        "Henriks",
        "Herbergen",
        "Herfølge",
        "Herholdts",
        "Herjedal",
        "Herlufsholm",
        "Hermods",
        "Herning",
        "Herslev",
        "Hesselø",
        "Hessens",
        "Hestemølle",
        "Hildurs",
        "Hillerød",
        "Hillerødmotoren",
        "Himmerlands",
        "Hindbær",
        "Hinde",
        "Hindustan",
        "Hirse",
        "Hirtshals",
        "Hjelms",
        "Hjertensfryds",
        "Hjerting",
        "Hjortdals",
        "Hjortholms",
        "Hjortø",
        "Hjørring",
        "Hobro",
        "Holbergs",
        "Holbæk",
        "Holbækmotoren",
        "Hollands",
        "Holmblads",
        "Holstebro",
        "Holsteinborg",
        "Holsteins",
        "Holte",
        "Hornbæk",
        "Hornemans",
        "Horsekilde",
        "Horsens",
        "Horserød",
        "Houmanns",
        "Hovedvagts",
        "Hovgaards",
        "Hovmester",
        "Hovmål",
        "Hulgårds",
        "Humlebæk",
        "Hustofte",
        "Husum",
        "Hvalsø",
        "Hvede",
        "Hveens",
        "Hvidbjerg",
        "Hvidkilde",
        "Hvidkløver",
        "Hvidtjørne",
        "Hyacint",
        "Hyldebær",
        "Hyltebjerg",
        "Hysken",
        "Hyttehus",
        "Händels",
        "Høffdings",
        "Høgholt",
        "Højbo",
        "Højdevangs",
        "Højde",
        "Højmose",
        "Højsager",
        "Højstrup",
        "Hørdums",
        "Hørhus",
        "Hørsholms",
        "Hørtofte",
        "Høsterkøb",
        "Høstgilde",
        "Høyens",
        "Håbets",
        "Ib Schønbergs",
        "Ilford",
        "India",
        "Industri",
        "Ingerslevs",
        "Ingolfs",
        "Ingrid Marie",
        "Iran",
        "Iris",
        "Irlands",
        "Irmingers",
        "Isafjords",
        "Islevhus",
        "Istanbul",
        "Isted",
        "Italiens",
        "Jagt",
        "James Grieve",
        "Jans",
        "Japan",
        "Java",
        "Jellinge",
        "Jemtelands",
        "Jena",
        "Jeppes",
        "Jerichaus",
        "Jernbane",
        "Bilbao",
        "Jernæble",
        "Jolle",
        "Jordbær",
        "Joris",
        "Judiths",
        "Jupiter",
        "Jyderup",
        "Jyllinge",
        "Jæger",
        "Jægersborg",
        "Jægerspris",
        "Kabbeleje",
        "Kaktus",
        "Kaldæa",
        "Kaliforniens",
        "Kalkbrænderihavns",
        "Kalø",
        "Kampmanns",
        "Kanada",
        "Kanonbåds",
        "Kansas",
        "Kansler",
        "Kapel",
        "Kapsel",
        "Kaprifolie",
        "Karens",
        "Karlskrona",
        "Karlslunde",
        "Karlstads",
        "Kasemat",
        "Kastanie",
        "Kastels",
        "Kastrup",
        "Katholm",
        "Katrinedals",
        "Kattegat",
        "Kattinge",
        "Kejser",
        "Keldsø",
        "Kentia",
        "Keplers",
        "Kerteminde",
        "Kildebrønde",
        "Kildevælds",
        "Kilholm",
        "Kina",
        "Kingos",
        "Kingston",
        "Kirkebjerg",
        "Kirkegårds",
        "Kirsteins",
        "Kirstinedals",
        "Kjeldsgårds",
        "Kjærstrup",
        "Klaipeda",
        "Klaksvigs",
        "Kleins",
        "Klerke",
        "Klingsey",
        "Klinte",
        "Klintholm",
        "Klitmøller",
        "Klostermarks",
        "Klosterris",
        "Kloster",
        "Klubiens",
        "Kløverblads",
        "Kløvermarks",
        "Knabro",
        "Knabstrup",
        "Knippelsbro",
        "Knivholt",
        "Knuthenborg",
        "Kolding",
        "Kompagni",
        "Kongebro",
        "Kongedybs",
        "Kongelunds",
        "Kongemarks",
        "Kongeæble",
        "Kongo",
        "Kongsdal",
        "Kongshøj",
        "Kongsted",
        "Korea",
        "Korfu",
        "Korinth",
        "Kornblomst",
        "Kornerup",
        "Kornskyld",
        "Korsager",
        "Kors",
        "Korsika",
        "Korsør",
        "Kortstilk",
        "Krabbesholm",
        "Kraftværks",
        "Krauses",
        "Kreta",
        "Krims",
        "Kristiania",
        "Krogager",
        "Krogerup",
        "Kroghs",
        "Krokodille",
        "Kronborg",
        "Kronprinsens",
        "Kronprinsesse",
        "Krudtløbs",
        "Krudtmøllegårds",
        "Krusemynte",
        "Kruså",
        "Krügers",
        "Krystal",
        "Kuglegårds",
        "Kuhlaus",
        "Kulbane",
        "Kurlands",
        "Kvintus",
        "Kvægtorvs",
        "Kvæsthus",
        "Küchlers",
        "Kyringe",
        "Kæmner",
        "Kærager",
        "Kærsanger",
        "Kærskifte",
        "Købmager",
        "Kålager",
        "Kaalunds",
        "Lager",
        "Lakse",
        "Landehjælp",
        "Landfoged",
        "Landgilde",
        "Landlyst",
        "Landsdommer",
        "Landskrona",
        "Landvindings",
        "Langager",
        "Langebro",
        "Langelinie",
        "Langhus",
        "Langkær",
        "Langø",
        "Laplands",
        "Larsbjørns",
        "Larslejs",
        "Laura",
        "Lautrups",
        "Lavendel",
        "Ledager",
        "Leifs",
        "Lejre",
        "Lemberg",
        "Lemnos",
        "Lerchenborg",
        "Lerfos",
        "Lergravs",
        "Letlands",
        "Lidemarks",
        "Liflands",
        "Lille Colbjørnsens",
        "Lille Farimags",
        "Lille Fredens",
        "Lille",
        "Lille Isted",
        "Lille Kannike",
        "Lille Kirke",
        "Lille Kongens",
        "Lille Strand",
        "Lille Søndervold",
        "Lille Thekla",
        "Lilliendals",
        "Limfjords",
        "Linde",
        "Lindenborg",
        "Lindenovs",
        "Lindgreens",
        "Lindholms",
        "Linnés",
        "Lipkes",
        "Liselund",
        "Livjæger",
        "Livorno",
        "Livø",
        "Lobelia",
        "Lodi",
        "Lombardi",
        "Lotus",
        "Lugano",
        "Lukretia",
        "Lundbyes",
        "Lundeborg",
        "Lundedals",
        "Lundehus",
        "Lundevangs",
        "Lundings",
        "Lundsfryd",
        "Lunds",
        "Lundtofte",
        "Lupin",
        "Lybæk",
        "Helsinki",
        "Lykkebo",
        "Lyneborg",
        "Lynette",
        "Lyngby",
        "Lyngholm",
        "Lyngvig",
        "Lynæs",
        "Lyon",
        "Lyrskov",
        "Lysefjords",
        "Lyshøj",
        "Lyshøjgårds",
        "Lystrup",
        "Læder",
        "Lærdals",
        "Lærke",
        "Læssøes",
        "Cork",
        "Løgstør",
        "Løgæble",
        "Løjtegårds",
        "Lønborg",
        "Løngang",
        "Lønstrup",
        "Løvetands",
        "P.D. Løvs",
        "Løv",
        "Magdelone",
        "Magister",
        "Mag",
        "Majrose",
        "Malakka",
        "Malmø",
        "Malta",
        "Mandals",
        "Mandel",
        "Mansas",
        "Mantua",
        "Manø",
        "Marathon",
        "Marbjerg",
        "Marengo",
        "Margretheholms",
        "Maribo",
        "Mariehamn",
        "Markmands",
        "Markskifte",
        "Mark",
        "Marmor",
        "Marsala",
        "Marskens",
        "Marstals",
        "Martha",
        "Masnedø",
        "Masteskurs",
        "Matthæus",
        "Meinungs",
        "Meklenborg",
        "Meldahls",
        "Mellemforts",
        "Mellemtofte",
        "Merløse",
        "Messina",
        "Metro",
        "Middelfart",
        "Middelgrunds",
        "Midgårds",
        "Mikkel Skovs",
        "Milano",
        "Milos",
        "Mimers",
        "Mimosa",
        "Mindstrup",
        "Minør",
        "Mirabelle",
        "Mitchells",
        "Mjøsens",
        "Molbechs",
        "Moldau",
        "Monrads",
        "Montagehals",
        "Montagne",
        "Morbær",
        "Morgendug",
        "Morsø",
        "Mosedal",
        "Mosel",
        "Mozarts",
        "Mullerup",
        "Murcia",
        "Murer",
        "Musholm",
        "Musvåge",
        "Mutzu",
        "Myggenæs",
        "Mysunde",
        "Møgeltønder",
        "Mølle",
        "Møllegårds",
        "C.F. Møllers",
        "Mønter",
        "Møntmester",
        "Mørkhøj",
        "Måge",
        "Mårum",
        "Nakskov",
        "Nannas",
        "Nansens",
        "Nattergale",
        "Neapel",
        "Nebraska",
        "Nelson Mandelas",
        "Nikolaj",
        "Nivå",
        "Njals",
        "Nokken Forn",
        "Nokken Hovedn",
        "Nokken Strand",
        "Nordbane",
        "Nordborg",
        "Nordby",
        "Nordfeld",
        "Skagerrak",
        "Nordhavns",
        "Nordlands",
        "Nordmarks",
        "Nordre",
        "Nordre Dige",
        "Nordre Fasan",
        "Nordre Frihavns",
        "Nordre Kongelunds",
        "Nordrup",
        "Nordsø",
        "Norges",
        "Norgesminde",
        "Normandi",
        "November",
        "Ny Adel",
        "Ny Blegdams",
        "Nyborg",
        "Nybo",
        "Nybro",
        "Ny",
        "Nygårds",
        "Ny Kongens",
        "Nyminde",
        "Nyrnberg",
        "Nyrops",
        "Nysted",
        "Nysø",
        "Ny Vester",
        "Ny Øster",
        "Nærum",
        "Næsbyholm",
        "Næstved",
        "Nøddebo",
        "Nøjsomheds",
        "Nøkkerose",
        "Nørager",
        "Nørre",
        "Nørrebro",
        "Nørre Farimags",
        "Nørre Sø",
        "Nørretofte",
        "Nørre Vold",
        "Obdams",
        "Ocean",
        "Odense",
        "Odins",
        "Odins Tvær",
        "Oehlenschlægers",
        "Offenbachs",
        "Oldermands",
        "Oldfux",
        "Oldenborg",
        "Olieblads",
        "Oliefabriks",
        "Oliemølle",
        "Olufs",
        "Olympos",
        "Omø",
        "Orgelbygger",
        "Orlogsværft",
        "Ottilia",
        "Otto Baches",
        "Ourø",
        "Overbys",
        "Overdrevs",
        "Overn Neden Vandet",
        "Overn Oven Vandet",
        "Overskous",
        "Oxford",
        "Padua",
        "Pakhus",
        "Palermo",
        "Pakkeri",
        "Palles",
        "Palnatokes",
        "Palæ",
        "Panums",
        "Parma",
        "Parnas",
        "Paros",
        "Pasteurs",
        "Peiters",
        "Per Henrik Lings",
        "Perlestikker",
        "Pernille",
        "Persiens",
        "Persille",
        "Peter Ipsens",
        "Petersborg",
        "Philip De Langes",
        "Pile",
        "Pindos",
        "Pistol",
        "Platan",
        "Polens",
        "Pommerns",
        "Pomona",
        "Poppel",
        "Portlands",
        "Portugals",
        "Postholder",
        "Pragtstjerne",
        "Primula",
        "Prinsesse",
        "Prisholm",
        "Provste",
        "Præstegårds",
        "Præstekær",
        "Præstemarks",
        "Præstø",
        "Prøvestens",
        "Puggaards",
        "Thomas Koppels",
        "Pæon",
        "Radise",
        "Rabarber",
        "Raffinaderi",
        "Ragna",
        "Ragnhild",
        "Rahbeks",
        "Ramløse",
        "Ramsings",
        "Ramunds",
        "Randbøl",
        "Randers",
        "Rantzaus",
        "Raunstrup",
        "Ravenna",
        "Ravneholms",
        "Ravnsborg",
        "Ravnsborg Tvær",
        "Rebekka",
        "Reberbane",
        "Rebild",
        "Rebslager",
        "Trelleborg",
        "Gdansk",
        "Reersø",
        "Refshale",
        "Refsnæs",
        "Regitse",
        "Reinette",
        "Rejsby",
        "Remise",
        "Rentemester",
        "Retort",
        "Reventlows",
        "Reverdils",
        "Reykjaviks",
        "Rialto",
        "Ribe",
        "Ridefoged",
        "Riga",
        "Rigens",
        "Rindby",
        "Ringholm",
        "Ringkøbing",
        "Ringsted",
        "Risager",
        "Risbyholm",
        "Rismose",
        "Rodos",
        "Romsdals",
        "Romsø",
        "Rosbæks",
        "Roselille",
        "Rosenborg",
        "Rosendals",
        "Rosen",
        "Rosenholms",
        "Rosenlunds",
        "Rosenvængets",
        "Rosenvængets Hoved",
        "Rosenørns",
        "Roshage",
        "Roskilde",
        "Rosmarin",
        "Rossinis",
        "Rostgaards",
        "Rostock",
        "Rothes",
        "Rovsings",
        "Rubikon",
        "Rubinola",
        "Rubinsteins",
        "Rugager",
        "Rughave",
        "Rug",
        "Rumæniens",
        "Rundholts",
        "Ruths",
        "Ryes",
        "Rygårds",
        "Rymarks",
        "Rysensteens",
        "Ryvangs",
        "Ræve",
        "Rødby",
        "Rødding",
        "Rødelands",
        "Røde Mellem",
        "Rødkilde",
        "Rødkløver",
        "Rødtjørne",
        "Rømers",
        "Rønnebær",
        "Rønne",
        "Rønnings",
        "Rørholms",
        "Rørmose",
        "Rørsanger",
        "Røså",
        "Rådhus",
        "Rådmands",
        "Rådvads",
        "Sadelmager",
        "Sakskøbing",
        "Salling",
        "Saltholms",
        "Saltø",
        "Samos",
        "Samsø",
        "Sandbjerg",
        "Sandbygård",
        "Sandhus",
        "Sankelmarks",
        "Sankt Jørgens",
        "Sassnitz",
        "Saxhøj",
        "Saxo",
        "Saxtorphs",
        "Scandia",
        "Schacks",
        "Scharlings",
        "Scherfigs",
        "Schleppegrells",
        "Schuberts",
        "Sejlklub",
        "Sejrø",
        "Seline",
        "Selsø",
        "Sele",
        "Serbiens",
        "Serridslev",
        "Shetlands",
        "Siam",
        "Sibberns",
        "Sibelius",
        "Siciliens",
        "Sigbrits",
        "Sigersted",
        "Signelil",
        "Sigurds",
        "Sigyns",
        "Siljan",
        "Silkeborg",
        "Silke",
        "Sions",
        "Sixtus",
        "Sjællands",
        "Skaffer",
        "Skanderborg",
        "Skarø",
        "Skelbæk",
        "Skelmose",
        "Skensved",
        "Skibelund",
        "Skinder",
        "Skipper Clements",
        "Skippinge",
        "Skjulhøj",
        "Skodsborg",
        "Skole",
        "Skoleholder",
        "Flyhangar",
        "Skotlands",
        "Skotterup",
        "Skoubo",
        "Skovbogårds",
        "Skovgaards",
        "Skovløber",
        "Skovstjerne",
        "Skudehavns",
        "Skydebane",
        "Skyggelunds",
        "Skytte",
        "Skyttegård",
        "Skåne",
        "Slagelse",
        "Slagtehus",
        "Slangerup",
        "Slejpners",
        "Slesvigs",
        "Slotsfoged",
        "Slots",
        "Slotsherrens",
        "Slotsholms",
        "Sluse",
        "Slutteri",
        "Slåen",
        "Smede",
        "Smyrna",
        "Smørum",
        "Smålands",
        "Snare",
        "H.C. Sneedorffs",
        "Sneppe",
        "Snertinge",
        "Snorres",
        "Sofie Brahes",
        "Sofie",
        "Sofienhøj",
        "Sognefjords",
        "Sokkelunds",
        "Solitude",
        "Solrød",
        "Solsikke",
        "Solskifte",
        "Soltofte",
        "Summerred",
        "Sommersted",
        "Sonnerup",
        "Sorgenfri",
        "Sorrento",
        "Sorø",
        "Southampton",
        "Spanager",
        "Spangbergs",
        "Spaniens",
        "Spanteloft",
        "Sparresholm",
        "Sparta",
        "Speditør",
        "Spinderi",
        "Spiræa",
        "Spontinis",
        "Sporemager",
        "Spøttrup",
        "Stadfeldts",
        "Stadil",
        "Stald",
        "Stampes",
        "Statholder",
        "Stavanger",
        "Stavnstrup",
        "Steenbergs",
        "Stefans",
        "Steins",
        "Stemanns",
        "Stenderup",
        "Sten",
        "Stenhugger",
        "Stenkløver",
        "Stenlands",
        "Stenlille",
        "Stenløse",
        "Stenmagle",
        "Stenos",
        "Stenrose",
        "Sternberg",
        "Stevns",
        "Stjerne",
        "Stockholms",
        "Stokhus",
        "Stokrose",
        "Stoltenbergs",
        "Storegårds",
        "Store Kannike",
        "Store Kirke",
        "Store Kongens",
        "Store Regne",
        "Store Strand",
        "Store Søndervold",
        "Storm",
        "Stradellas",
        "Strandager",
        "Strand",
        "Strandlods",
        "Stranden",
        "Stratford",
        "Strauss",
        "Strickers",
        "Strindbergs",
        "Struensee",
        "Strynø",
        "Strødam",
        "Stubbeløb",
        "Stubmølle",
        "Studie",
        "Studsgaards",
        "Sturlas",
        "Stære",
        "Støberi",
        "Støvnæs",
        "Støvring",
        "Suensons",
        "Suhms",
        "Sumatra",
        "Sundbygårds",
        "Sundby Park",
        "Sundbyvester",
        "Sundeveds",
        "Sundholms",
        "Sundkrogs",
        "Svane",
        "Svanemølle",
        "Svankær",
        "Svendborg",
        "Svends",
        "Svenstrup",
        "Sverrigs",
        "Svogerslev",
        "Sværte",
        "Sydhavns",
        "Sydløbs",
        "Sylvia",
        "Syriens",
        "Syvens",
        "Syvstens",
        "Sæby",
        "Sæbyholms",
        "Sætersdal",
        "Søfort",
        "Søllerød",
        "Sølunds",
        "Sølv",
        "Sønderborg",
        "Søndermarks",
        "Søndervangs",
        "Søndervig",
        "Søndre",
        "Søndre Fasan",
        "Søren Norbys",
        "Sørup",
        "Saabyes",
        "Taffelæble",
        "Tagens",
        "Takkelads",
        "Takkelloft",
        "Tallinn",
        "Tartinis",
        "Teglbrænder",
        "Teglgård",
        "Teglholm",
        "Teglholms",
        "Teglholm Tvær",
        "Teglstrup",
        "Teglværks",
        "Telemarks",
        "Tersløse",
        "Theis",
        "Thekla",
        "Thingvalla",
        "Thora",
        "Thors",
        "Thorshavns",
        "Thorsminde",
        "Thorupgård",
        "Thorups",
        "Thurebyholm",
        "Thyras",
        "Thyregods",
        "Thy",
        "Tibirke",
        "Tietgens",
        "Tiger",
        "Tikøb",
        "Timians",
        "Tingskifte",
        "Tingskriver",
        "Ting",
        "Tipsager",
        "Tirsbæk",
        "Titan",
        "Tjæreby",
        "Tjørne",
        "Tjørnelunds",
        "Todes",
        "Toftager",
        "Toftebakke",
        "Toftegårds",
        "Toftøje",
        "Toldbod",
        "Toldskriver",
        "Tomat",
        "Tomsgårds",
        "Tonemester",
        "Torbenfeldt",
        "Torben Oxes",
        "Tordenskjolds",
        "Torfa",
        "Tornebuske",
        "Tornsanger",
        "Torve",
        "Toskifte",
        "Tovelille",
        "Tovværk",
        "Tranehave",
        "Trane",
        "Trangravs",
        "Traps",
        "Trekløver",
        "Trekroner",
        "Trepkas",
        "Troja",
        "Tromsø",
        "Trondhjems",
        "Tryggevælde",
        "Trøjborg",
        "Tschernings",
        "Tuborg",
        "Tudseminde",
        "Tudskær",
        "Tuelands",
        "Tulipan",
        "Tullins",
        "Turesens",
        "Tustrup",
        "Tuxens",
        "Tværager",
        "Kiel",
        "Tybjerg",
        "Tyborøn",
        "Tycho Brahes",
        "Tyrols",
        "Tyttebær",
        "Tøjhus",
        "Tøjmester",
        "Tølløse",
        "Tømmergravs",
        "Tømrer",
        "Tøndebinder",
        "Tønder",
        "Tønnes",
        "Tårnblæser",
        "Tårnholms",
        "Tårnhus",
        "Tåsinge",
        "Tåstrup",
        "Udby",
        "Uffes",
        "Uggerløse",
        "Ugle",
        "Ullerup",
        "Ulrik Birchs",
        "Ulriksdal",
        "Ulvefod",
        "Ulvsund",
        "Ungarns",
        "Uplands",
        "Upsala",
        "Ural",
        "Urbans",
        "Utterslev",
        "Wagners",
        "Vagtmester",
        "Valborg",
        "Valbygårds",
        "Valby Kirke",
        "Valby Lang",
        "Valby Maskinfabriks",
        "Valby Torve",
        "Valdemars",
        "Valgårds",
        "Valhals",
        "Valhøj",
        "Valkendorfs",
        "Valkyrie",
        "Vallekilde",
        "Vallø",
        "Valmue",
        "Valnødde",
        "Vangehus",
        "Vangs",
        "Vangså",
        "Vanløse",
        "Vanløse By",
        "Varde",
        "Vasby",
        "Vatna",
        "Webers",
        "Vedbæk",
        "Weidekamps",
        "Weimar",
        "Vejlands",
        "Vejle",
        "Vejrø",
        "Veksø",
        "Venders",
        "Vendsyssel",
        "Venedig",
        "Vennely",
        "Venneminde",
        "Venø",
        "Veras",
        "Verdis",
        "Vermlands",
        "Vermunds",
        "Verona",
        "Wessels",
        "Vestager",
        "Vestbane",
        "Vesterbro",
        "Vester Farimags",
        "Vesterfælled",
        "Vester",
        "Vestergårds",
        "Vesterhavs",
        "Vestermarks",
        "Vester Sø",
        "Vestervig",
        "Vester Vold",
        "Vestmanna",
        "Vestre",
        "Vestre Kirkegårds",
        "Vestre Tegl",
        "Weyses",
        "Vibeke",
        "Vibe",
        "Viborg",
        "Wibrandts",
        "Wiedewelts",
        "Vigerslev",
        "Vigerslev Gods",
        "Wiinblads",
        "Viktoria",
        "Vildande",
        "Wilders",
        "Vilhelm Thomsens",
        "Willemoes",
        "Willums",
        "Vindebro",
        "Vindinge",
        "Vindmølle",
        "Vindrue",
        "Windsor",
        "Vingård",
        "Visby",
        "Wittenberg",
        "Vognborg",
        "Vognmager",
        "Vogter",
        "Voldmester",
        "Volos",
        "Wolters",
        "Vordingborg",
        "Vulkan",
        "Væbner",
        "Værksted",
        "Værnedams",
        "Væver",
        "Vølunds",
        "Vånings",
        "Yderlands",
        "Yduns",
        "Ystad",
        "Zinns",
        "Æbelø",
        "Æble",
        "Ægina",
        "Ægirs",
        "Ærtebjerg",
        "Ærtelands",
        "Ærte",
        "Ætna",
        "Ølands",
        "Øresund Park",
        "Øresundsmotoren",
        "Øresunds",
        "Ørevads",
        "Ørhage",
        "Ørholm",
        "Ørne",
        "Øsels",
        "Østbane",
        "Øster",
        "Østerbro",
        "Østerdals",
        "Øster Farimags",
        "Østergårds",
        "Øster Sø",
        "Østersø",
        "Øster Vold",
        "Østre Tegl",
        "Østrigs",
        "Åbakke",
        "Åbjerg",
        "Ådals",
        "Å",
        "Ågerup",
        "Åkande",
        "Ålands",
        "Ålborg",
        "Ålekiste",
        "Ålholm",
        "Ålstrup",
        "Åløkke",
        "Aarestrups",
        "Århus",
        "Mælkeen",
        "Løvstikke",
        "Murmansk",
        "Antwerpen",
        "Travemünde",
        "Bohrs",
        "Cylinder",
        "Støbegods",
        "Pladehals",
        "Kul",
        "Diesel",
        "Gloster",
        "Burgundy",
        "Paradisæble",
    )

    street_suffixes = (
        # gade and vej are the most common so they should be oversampled
        "gade",
        "gade",
        "stræde",
        "vej",
        "vej",
        "vej",
        " Allé",
    )

    address_formats = ("{{street_address}}\n{{postcode}} {{city}}",)

    # Postcode should be formatted as described in http://www.nr.dk/danmark.html
    postcode_formats = ("%###",)

    city_formats = ("{{city_name}}",)

    cities = (
        "Allinge",
        "Allingåbro",
        "Almind",
        "Anholt",
        "Ans by",
        "Ansager",
        "Arden",
        "Askeby",
        "Asnæs",
        "Asperup",
        "Assens",
        "Asaa",
        "Augustenborg",
        "Aulum",
        "Auning",
        "Bagenkop",
        "Bagsværd",
        "Balle",
        "Ballerup",
        "Bandholm",
        "Barrit",
        "Beder",
        "Bedsted Thy",
        "Bevtoft",
        "Billum",
        "Billund",
        "Bindslev",
        "Birkerød",
        "Bjerringbro",
        "Bjert",
        "Bjæverskov",
        "Blokhus",
        "Blommenslyst",
        "Blåvand",
        "Boeslunde",
        "Bogense",
        "Bogø By",
        "Bolderslev",
        "Bording",
        "Borre",
        "Borup",
        "Brabrand",
        "Bramming",
        "Brande",
        "Branderup",
        "Bredebro",
        "Bredsten",
        "Brenderup",
        "Broager",
        "Broby",
        "Brovst",
        "Bryrup",
        "Brædstrup",
        "Brøndby",
        "Brøndby Strand",
        "Brønderslev",
        "Brønshøj",
        "Brørup",
        "Bylderup-Bov",
        "Bække",
        "Bækmarksbro",
        "Bælum",
        "Børkop",
        "Bøvlingbjerg",
        "Charlottenlund",
        "Christiansfeld",
        "Dalby",
        "Dalmose",
        "Dannemare",
        "Daugård",
        "Dianalund",
        "Dragør",
        "Dronninglund",
        "Dronningmølle",
        "Dybvad",
        "Ebberup",
        "Ebeltoft",
        "Egernsund",
        "Egtved",
        "Egå",
        "Ejby",
        "Ejstrupholm",
        "Engesvang",
        "Errindlev",
        "Erslev",
        "Esbjerg",
        "Eskebjerg",
        "Eskilstrup",
        "Espergærde",
        "Fakse",
        "Fakse Ladeplads",
        "Fanø",
        "Farsø",
        "Farum",
        "Fejø",
        "Ferritslev Fyn",
        "Fjenneslev",
        "Fjerritslev",
        "Flemming",
        "Fredensborg",
        "Fredericia",
        "Frederiksberg",
        "Frederikshavn",
        "Frederikssund",
        "Frederiksværk",
        "Frørup",
        "Frøstrup",
        "Fuglebjerg",
        "Fur",
        "Føllenslev",
        "Føvling",
        "Faaborg",
        "Fårevejle",
        "Fårup",
        "Fårvang",
        "Gadbjerg",
        "Gadstrup",
        "Galten",
        "Gandrup",
        "Gedser",
        "Gedsted",
        "Gedved",
        "Gelsted",
        "Gentofte",
        "Gesten",
        "Gilleleje",
        "Gislev",
        "Gislinge",
        "Gistrup",
        "Give",
        "Gjerlev",
        "Gjern",
        "Glamsbjerg",
        "Glejbjerg",
        "Glesborg",
        "Glostrup",
        "Glumsø",
        "Gram",
        "Gredstedbro",
        "Grenaa",
        "Greve",
        "Greve Strand",
        "Grevinge",
        "Grindsted",
        "Græsted",
        "Gråsten",
        "Gudbjerg",
        "Gudhjem",
        "Gudme",
        "Guldborg",
        "Gørding",
        "Gørlev",
        "Gørløse",
        "Haderslev",
        "Haderup",
        "Hadsten",
        "Hadsund",
        "Hagersten",
        "Hals",
        "Hammel",
        "Hampen",
        "Hanstholm",
        "Harboøre",
        "Harlev",
        "Harndrup",
        "Harpelunde",
        "Hasle",
        "Haslev",
        "Hasselager",
        "Havdrup",
        "Havndal",
        "Hedehusene",
        "Hedensted",
        "Hejls",
        "Hejnsvig",
        "Hellebæk",
        "Hellerup",
        "Helsinge",
        "Helsingør",
        "Hemmet",
        "Henne",
        "Herfølge",
        "Herlev",
        "Herlufmagle",
        "Herning",
        "Hesselager",
        "Hillerød",
        "Hinnerup",
        "Hirtshals",
        "Hjallerup",
        "Hjerm",
        "Hjortshøj",
        "Hjørring",
        "Hobro",
        "Holbæk",
        "Holeby",
        "Holme-Olstrup",
        "Holstebro",
        "Holsted",
        "Holte",
        "Horbelev",
        "Hornbæk",
        "Hornslet",
        "Hornsyld",
        "Horsens",
        "Horslunde",
        "Hovborg",
        "Hovedgård",
        "Humble",
        "Humlebæk",
        "Hundested",
        "Hundslund",
        "Hurup Thy",
        "Hvalsø",
        "Hvide Sande",
        "Hvidovre",
        "Højbjerg",
        "Højby",
        "Højer",
        "Højslev",
        "Høng",
        "Hørning",
        "Hørsholm",
        "Hørve",
        "Haarby",
        "Hårlev",
        "Idestrup",
        "Ikast",
        "Ishøj",
        "Janderup",
        "Jelling",
        "Jerslev",
        "Jerslev",
        "Jerup",
        "Jordrup",
        "Juelsminde",
        "Jyderup",
        "Jyllinge",
        "Jystrup",
        "Jægerspris",
        "Kalundborg",
        "Kalvehave",
        "Karby",
        "Karise",
        "Karlslunde",
        "Karrebæksminde",
        "Karup",
        "Kastrup",
        "Kerteminde",
        "Kettinge",
        "Kibæk",
        "Kirke Eskilstrup",
        "Kirke Hyllinge",
        "Kirke Såby",
        "Kjellerup",
        "Klampenborg",
        "Klarup",
        "Klemensker",
        "Klippinge",
        "Klovborg",
        "Knebel",
        "Kokkedal",
        "Kolding",
        "Kolind",
        "Kongens Lyngby",
        "Kongerslev",
        "Korsør",
        "Kruså",
        "Kvistgård",
        "Kværndrup",
        "København",
        "Køge",
        "Langebæk",
        "Langeskov",
        "Langå",
        "Lejre",
        "Lem",
        "Lemming",
        "Lemvig",
        "Lille Skensved",
        "Lintrup",
        "Liseleje",
        "Lundby",
        "Lunderskov",
        "Lynge",
        "Lystrup",
        "Læsø",
        "Løgstrup",
        "Løgstør",
        "Løgumkloster",
        "Løkken",
        "Løsning",
        "Låsby",
        "Malling",
        "Mariager",
        "Maribo",
        "Marslev",
        "Marstal",
        "Martofte",
        "Melby",
        "Mern",
        "Mesinge",
        "Middelfart",
        "Millinge",
        "Morud",
        "Munke Bjergby",
        "Munkebo",
        "Møldrup",
        "Mørke",
        "Mørkøv",
        "Måløv",
        "Mårslet",
        "Nakskov",
        "Nexø",
        "Nibe",
        "Nimtofte",
        "Nivå",
        "Nordborg",
        "Nyborg",
        "Nykøbing F",
        "Nykøbing M",
        "Nykøbing Sj",
        "Nyrup",
        "Nysted",
        "Nærum",
        "Næstved",
        "Nørager",
        "Nørre Alslev",
        "Nørre Asmindrup",
        "Nørre Nebel",
        "Nørre Snede",
        "Nørre Aaby",
        "Nørreballe",
        "Nørresundby",
        "Odder",
        "Odense",
        "Oksbøl",
        "Otterup",
        "Oure",
        "Outrup",
        "Padborg",
        "Pandrup",
        "Præstø",
        "Randbøl",
        "Randers",
        "Ranum",
        "Rask Mølle",
        "Redsted",
        "Regstrup",
        "Ribe",
        "Ringe",
        "Ringkøbing",
        "Ringsted",
        "Risskov",
        "Roskilde",
        "Roslev",
        "Rude",
        "Rudkøbing",
        "Ruds Vedby",
        "Rungsted Kyst",
        "Ry",
        "Rynkeby",
        "Ryomgård",
        "Ryslinge",
        "Rødby",
        "Rødding",
        "Rødekro",
        "Rødkærsbro",
        "Rødovre",
        "Rødvig Stevns",
        "Rømø",
        "Rønde",
        "Rønne",
        "Rønnede",
        "Rørvig",
        "Sabro",
        "Sakskøbing",
        "Saltum",
        "Samsø",
        "Sandved",
        "Sejerø",
        "Silkeborg",
        "Sindal",
        "Sjællands Odde",
        "Sjølund",
        "Skagen",
        "Skals",
        "Skamby",
        "Skanderborg",
        "Skibby",
        "Skive",
        "Skjern",
        "Skodsborg",
        "Skovlunde",
        "Skælskør",
        "Skærbæk",
        "Skævinge",
        "Skødstrup",
        "Skørping",
        "Skårup",
        "Slagelse",
        "Slangerup",
        "Smørum",
        "Snedsted",
        "Snekkersten",
        "Snertinge",
        "Solbjerg",
        "Solrød Strand",
        "Sommersted",
        "Sorring",
        "Sorø",
        "Spenstrup",
        "Spjald",
        "Sporup",
        "Spøttrup",
        "Stakroge",
        "Stege",
        "Stenderup",
        "Stenlille",
        "Stenløse",
        "Stenstrup",
        "Stensved",
        "Stoholm",
        "Stokkemarke",
        "Store Fuglede",
        "Store Heddinge",
        "Store Merløse",
        "Storvorde",
        "Stouby",
        "Strandby",
        "Struer",
        "Strøby",
        "Stubbekøbing",
        "Støvring",
        "Suldrup",
        "Sulsted",
        "Sunds",
        "Svaneke",
        "Svebølle",
        "Svendborg",
        "Svenstrup",
        "Svinninge",
        "Sydals",
        "Sæby",
        "Søborg",
        "Søby Ærø",
        "Søllested",
        "Sønder Felding",
        "Sønder Omme",
        "Sønder Stenderup",
        "Sønderborg",
        "Søndersø",
        "Sørvad",
        "Tappernøje",
        "Tarm",
        "Terndrup",
        "Them",
        "Thisted",
        "Thorsø",
        "Thyborøn",
        "Thyholm",
        "Tikøb",
        "Tilst",
        "Tim",
        "Tinglev",
        "Tistrup",
        "Tisvildeleje",
        "Tjele",
        "Tjæreborg",
        "Toftlund",
        "Tommerup",
        "Toreby",
        "Torrig",
        "Tranbjerg J",
        "Tranekær",
        "Trige",
        "Trustrup",
        "Tureby",
        "Tylstrup",
        "Tølløse",
        "Tønder",
        "Tørring",
        "Tårs",
        "Taastrup",
        "Ugerløse",
        "Uldum",
        "Ulfborg",
        "Ullerslev",
        "Ulstrup",
        "Vadum",
        "Valby",
        "Vallensbæk",
        "Vallensbæk Strand",
        "Vamdrup",
        "Vandel",
        "Vanløse",
        "Varde",
        "Vedbæk",
        "Veflinge",
        "Vejby",
        "Vejen",
        "Vejers Strand",
        "Vejle",
        "Vejle Øst",
        "Vejstrup",
        "Veksø Sjælland",
        "Vemb",
        "Vemmelev",
        "Vesløs",
        "Vestbjerg",
        "Vester Skerninge",
        "Vesterborg",
        "Vestervig",
        "Viborg",
        "Viby J",
        "Viby Sjælland",
        "Videbæk",
        "Vig",
        "Vildbjerg",
        "Vils",
        "Vinderup",
        "Vipperød",
        "Virum",
        "Vissenbjerg",
        "Viuf",
        "Vodskov",
        "Vojens",
        "Vonge",
        "Vorbasse",
        "Vordingborg",
        "Vrå",
        "Væggerløse",
        "Værløse",
        "Ærøskøbing",
        "Ølgod",
        "Ølsted",
        "Ølstykke",
        "Ørbæk",
        "Ørnhøj",
        "Ørsted",
        "Ørum Djurs",
        "Østbirk",
        "Øster Assels",
        "Øster Ulslev",
        "Øster Vrå",
        "Østermarie",
        "Aabenraa",
        "Aabybro",
        "Åbyhøj",
        "Aakirkeby",
        "Aalborg",
        "Ålbæk",
        "Aalestrup",
        "Ålsgårde",
        "Århus",
        "Årre",
        "Aars",
        "Årslev",
        "Aarup",
    )

    countries = (
        "Afghanistan",
        "Albanien",
        "Algeriet",
        "Andorra",
        "Angola",
        "Antigua og Barbuda",
        "Argentina",
        "Armenien",
        "Aserbajdsjan",
        "Australien",
        "Bahamas",
        "Bahrain",
        "Bangladesh",
        "Barbados",
        "Belgien",
        "Belize",
        "Benin",
        "Bermuda",
        "Bhutan",
        "Bolivia",
        "Bosnien og Hercegovina",
        "Botswana",
        "Brasilien",
        "Brunei",
        "Bulgarien",
        "Burkina Faso",
        "Burma",
        "Burundi",
        "Cambodja",
        "Cameroun",
        "Canada",
        "Centralafrikanske Republik",
        "Chile",
        "Colombia",
        "Comorerne",
        "Republikken Congo",
        "Den Demokratiske Republik Congo",
        "Costa Rica",
        "Cuba",
        "Cypern",
        "Danmark",
        "Djibouti",
        "Dominica",
        "Dominikanske Republik",
        "Ecuador",
        "Egypten",
        "El Salvador",
        "Elfenbenskysten",
        "Eritrea",
        "Estland",
        "Etiopien",
        "Fiji",
        "Filippinerne",
        "Finland",
        "Forenede Arabiske Emirater",
        "Frankrig",
        "Færøerne",
        "Gabon",
        "Gambia",
        "Georgien",
        "Ghana",
        "Grenada",
        "Grækenland",
        "Grønland",
        "Guatemala",
        "Guinea",
        "Guinea-Bissau",
        "Guyana",
        "Fransk Guiana",
        "Haiti",
        "Holland",
        "Honduras",
        "Hviderusland",
        "Indien",
        "Indonesien",
        "Irak",
        "Iran",
        "Irland",
        "Island",
        "Israel",
        "Italien",
        "Jamaica",
        "Japan",
        "Jordan",
        "Kap Verde",
        "Kasakhstan",
        "Kenya",
        "Kina",
        "Kirgisistan",
        "Kiribati",
        "Kroatien",
        "Kuwait",
        "Laos",
        "Lesotho",
        "Letland",
        "Libanon",
        "Liberia",
        "Libyen",
        "Liechtenstein",
        "Litauen",
        "Luxembourg",
        "Madagaskar",
        "Makedonien",
        "Malawi",
        "Malaysia",
        "Maldiverne",
        "Mali",
        "Malta",
        "Marokko",
        "Marshall-øerne",
        "Mauretanien",
        "Mauritius",
        "Mexico",
        "Mikronesien",
        "Moldova",
        "Monaco",
        "Mongoliet",
        "Montenegro",
        "Mozambique",
        "Myanmar",
        "Namibia",
        "Nauru",
        "Nederlandske Antiller",
        "Nepal",
        "New Zealand",
        "Nicaragua",
        "Niger",
        "Nigeria",
        "Niue",
        "Nordkorea",
        "Norge",
        "Oman",
        "Pakistan",
        "Palau",
        "Palæstinensisk Selvstyreområde",
        "Panama",
        "Papua Ny Guinea",
        "Paraguay",
        "Peru",
        "Pitcairn",
        "Polen",
        "Portugal",
        "Puerto Rico",
        "Qatar",
        "Rumænien",
        "Rusland",
        "Rwanda",
        "Saint Kitts and Nevis",
        "Saint Lucia",
        "Saint Vincent og Grenadinerne",
        "Salomonøerne",
        "Samoa",
        "San Marino",
        "São Tomé og Príncipe",
        "Saudi-Arabien",
        "Schweiz",
        "Senegal",
        "Serbien",
        "Seychellerne",
        "Sierra Leone",
        "Singapore",
        "Slovakiet",
        "Slovenien",
        "Somalia",
        "Spanien",
        "Sri Lanka",
        "Storbritannien",
        "Sudan",
        "Surinam",
        "Sverige",
        "Swaziland",
        "Sydafrika",
        "Sydkorea",
        "Syrien",
        "Tadsjikistan",
        "Taiwan",
        "Tanzania",
        "Tchad",
        "Thailand",
        "Tjekkiet",
        "Tjetjenien",
        "Togo",
        "Tonga",
        "Trinidad og Tobago",
        "Tunesien",
        "Turkmenistan",
        "Tuvalu",
        "Tyrkiet",
        "Tyskland",
        "Uganda",
        "Ukraine",
        "Ungarn",
        "Uruguay",
        "USA",
        "Usbekistan",
        "Vanuatu",
        "Vatikanstaten",
        "Venezuela",
        "Vestsahara",
        "Vietnam",
        "Yemen",
        "Zambia",
        "Zimbabwe",
        "Ækvatorialguinea",
        "Østrig",
        "Østtimor",
    )

    # Known as regions in Denmark
    states = (
        "Hovedstaden",
        "Midtjylland",
        "Nordjylland",
        "Sjælland",
        "Syddanmark",
    )

    def dk_street_name(self) -> str:
        """
        This returns the name of a street, without any suffix.
        """
        return self.random_element(self.street_names)

    def city_name(self) -> str:
        return self.random_element(self.cities)

    def administrative_unit(self) -> str:
        return self.random_element(self.states)

    state = administrative_unit



"""
使用allure需要下载，并配置到环境变量，下载地址：https://github.com/allure-framework/allure2/releases
"""
import json
import os
import allure
import pytest
import inspect as sys_inspect


# 报告模块
def feature(text):
    return allure.feature(text)


# 报告子模块
def story(text):
    return allure.story(text)


# 报告子模块，兼容历史版本
def module(text):
    return allure.story(text)


# 报告标题
def title(text):
    return allure.title(text)


# 报告步骤
def step(text):
    return allure.step(text)


def order(index):
    """
    指定用例执行顺序
    doc: https://blog.csdn.net/weixin_43880991/article/details/116221362
    """
    return pytest.mark.run(order=index)


def depend(*args, **kwargs):
    """
    设置用例依赖关系
    doc: https://www.cnblogs.com/se7enjean/p/13513131.html
    """
    return pytest.mark.depenency(*args, **kwargs)


# 参数化数据
def data(*args, **kwargs):
    """
    @data('key1,key2', [[value1, value2], [value3, value4]])
    """
    return pytest.mark.parametrize(*args, **kwargs)


# 从json文件获取数据进行参数化
def file_data(data_key=None, file_name=None):
    """
    @file_data('key1,key2', 'data.json')
    data.json
        {
            "key1,key2": [[value1, value2], [value3, value4]]
        }
    """
    # 获取被装饰方法的目录
    stack_t = sys_inspect.stack()
    ins = sys_inspect.getframeinfo(stack_t[1][0])
    file_dir = os.path.dirname(os.path.abspath(ins.filename))
    parent_dir = os.path.dirname(file_dir)
    parent_dir_dir = os.path.dirname(parent_dir)
    parent_dir_dir_dir = os.path.dirname(parent_dir_dir)

    # 判断当前、父目录、爷目录、太爷爷目录下的test_data目录下是否有file_name文件
    path_list = [file_dir, parent_dir, parent_dir_dir, parent_dir_dir_dir]
    file_path = None
    full_list = []
    for _path in path_list:
        full_path = os.path.join(_path, 'test_data', file_name)
        full_list.append(full_path)
        # print(full_path)
        if os.path.isfile(full_path) is True:
            file_path = full_path
            break
    if file_path is None:
        raise Exception(f"can not found {file_name} in {full_list}")

    # 根据文件路径以及key获取文件中的数据，暂时只支持json文件
    with open(file_path, 'r', encoding='utf-8') as f:
        json_data = json.load(f)

    content = json_data[data_key]
    return data(data_key, content)



import time
from qrunner.utils.faker import Faker
from qrunner.utils.log import logger

"""
可用的mock方法见：https://zhuanlan.zhihu.com/p/87203290
"""


# 兼容老版本
mock = Faker('zh_CN')
mock_en = Faker()


# 建议使用
class RandomData(Faker):
    """随机数据"""

    def __init__(self, language='中文'):
        if language == '中文':
            locale = 'zh_CN'
            super().__init__(locale=locale)
        elif language == '英文':
            super().__init__()
        else:
            logger.debug(f'暂不支持这种语言-{language}')

    def get_word(self):
        """随机词语"""
        return self.word()

    def get_words(self):
        """随机词语列表"""
        return self.words()

    def get_phone(self):
        """随机手机号"""
        return self.phone_number()

    def get_company_name(self):
        """随机公司名"""
        return self.company()

    def get_name(self):
        """随机人名"""
        return self.name()

    def get_timezone(self):
        """随机时区"""
        return self.timezone()

    def get_date(self):
        """随机日期"""
        return self.date()

    def get_number(self, length=3):
        """随机数"""
        return self.random_number(digits=length)

    def get_ssn(self):
        """随机身份证号"""
        return self.ssn()

    def get_email(self):
        """随机邮箱"""
        return self.email()

    def get_url(self):
        """随机url地址"""
        return self.url()


class CommonData:
    """其它常用数据"""

    @staticmethod
    def get_timestamp(length=None) -> str:
        """获取当前时间戳"""
        timestamp = str(int(time.time()))
        if length is None:
            return timestamp
        else:
            return timestamp.ljust(length, '0')

    @staticmethod
    def get_date(_format="%Y-%m-%d"):
        """获取当前日期"""
        return time.strftime(_format)

    @staticmethod
    def get_now_time(_format="%Y-%m-%d %H:%M:%S"):
        """获取当前时间"""
        return time.strftime(_format)















import json
import logging
import os
import sys
import time
import allure
from baseImage import Image, Rect
from selenium import webdriver
from selenium.webdriver import DesiredCapabilities
from selenium.webdriver import ChromeOptions
from selenium.webdriver import FirefoxOptions
from qrunner.utils.decorate import step
from qrunner.utils.webdriver_manager_extend import ChromeDriverManager
from webdriver_manager.firefox import GeckoDriverManager
from webdriver_manager.microsoft import IEDriverManager
from webdriver_manager.microsoft import EdgeChromiumDriverManager
from webdriver_manager.opera import OperaDriverManager
from qrunner.utils.log import logger
from qrunner.utils.config import conf
from urllib import parse
from selenium.webdriver.remote.remote_connection import LOGGER

LOGGER.setLevel(logging.WARNING)


# def relaunch(func):
#     """
#     重启chromedriver的装饰器
#     :param func: 待包裹的函数
#     :return:
#     """
#     def wrapper(self, *args, **kwargs):
#         try:
#             return func(self, *args, **kwargs)
#         except requests.exceptions.ConnectionError as _:
#             logger.warning("web driver error, relaunch now.")
#             name = conf.get_name('web', 'browser_name')
#             self.d = BrowserDriver(name)
#     return wrapper


class ChromeConfig:
    headless = False
    options = None
    command_executor = ""


class FirefoxConfig:
    headless = False
    options = None
    command_executor = ""


class IEConfig:
    command_executor = ""


class EdgeConfig:
    command_executor = ""


class OperaConfig:
    command_executor = ""


class SafariConfig:
    executable_path = "/usr/bin/safaridriver"
    command_executor = ""


class Browser(object):
    """
    根据关键词初始化浏览器操作句柄，
    如'chrome、google chrome、gc'代表chrome浏览器，
    如'firefox、ff'代表火狐浏览器，
    如'internet explorer、ie、IE'代表ie浏览器，
    如'edge'代表edge浏览器，
    如'opera'代表opera浏览器，
    如'safari'代表safari浏览器
    """
    name = None

    def __new__(cls, name=None):
        cls.name = name

        if (cls.name is None) or (cls.name in ["chrome", "google chrome", "gc"]):
            return cls.chrome()
        elif cls.name in ['internet explorer', 'ie', 'IE']:
            return cls.ie()
        elif cls.name in ['firefox', 'ff']:
            return cls.firefox()
        elif cls.name == 'edge':
            return cls.edge()
        elif cls.name == 'opera':
            return cls.opera()
        elif cls.name == 'safari':
            return cls.safari()
        raise NameError(f"Not found {cls.name} browser")

    @staticmethod
    def chrome():
        if ChromeConfig.command_executor != "":
            return webdriver.Remote(command_executor=ChromeConfig.command_executor,
                                    desired_capabilities=DesiredCapabilities.CHROME.copy())

        if ChromeConfig.options is None:
            chrome_options = ChromeOptions()
        else:
            chrome_options = ChromeConfig.options
        if ChromeConfig.headless is True:
            chrome_options.add_argument('--headless')

        chrome_options.add_argument("--incognito")  # 隐身模式
        chrome_options.add_experimental_option("excludeSwitches",
                                               ['enable-automation'])  # 去除自动化控制提示
        # prefs = {'profile.managed_default_content_settings.images': 2}  # 不加载图片
        # prefs = {'profile.managed_default_content_settings.javascript': 2}  # 不加载js
        # chrome_options.add_experimental_option('prefs', prefs)
        chrome_options.page_load_strategy = 'normal'  # 设置页面加载策略
        driver = webdriver.Chrome(options=chrome_options,
                                  executable_path=ChromeDriverManager().install(),
                                  )
        if ChromeConfig.headless is True:
            driver.set_window_size(1920, 1080)
        else:
            driver.maximize_window()
        # driver.execute_cdp_cmd("Page.addScriptToEvaluateOnNewDocument", {
        #     "source": """
        #         Object.defineProperty(navigator, 'webdriver', {
        #         get: () => undefined
        #         })"""
        # })
        return driver

    @staticmethod
    def firefox():
        if FirefoxConfig.command_executor != "":
            return webdriver.Remote(command_executor=FirefoxConfig.command_executor,
                                    desired_capabilities=DesiredCapabilities.FIREFOX.copy())

        if FirefoxConfig.options is not None:
            firefox_options = FirefoxOptions()
        else:
            firefox_options = FirefoxConfig.options
        if FirefoxConfig.headless is True:
            firefox_options.headless = True

        driver = webdriver.Firefox(options=firefox_options,
                                   executable_path=GeckoDriverManager().install())
        if FirefoxConfig.headless is True:
            driver.set_window_size(1920, 1080)
        else:
            driver.maximize_window()
        return driver

    @staticmethod
    def ie():
        if IEConfig.command_executor != "":
            return webdriver.Remote(command_executor=IEConfig.command_executor,
                                    desired_capabilities=DesiredCapabilities.INTERNETEXPLORER.copy())
        driver = webdriver.Ie(executable_path=IEDriverManager().install())
        driver.maximize_window()
        return driver

    @staticmethod
    def edge():
        if EdgeConfig.command_executor != "":
            return webdriver.Remote(command_executor=EdgeConfig.command_executor,
                                    desired_capabilities=DesiredCapabilities.EDGE.copy())
        driver = webdriver.Edge(executable_path=EdgeChromiumDriverManager(log_level=1).install())
        driver.maximize_window()
        return driver

    @staticmethod
    def opera():
        if OperaConfig.command_executor != "":
            return webdriver.Remote(command_executor=OperaConfig.command_executor,
                                    desired_capabilities=DesiredCapabilities.OPERA.copy())
        driver = webdriver.Opera(executable_path=OperaDriverManager().install())
        driver.maximize_window()
        return driver

    @staticmethod
    def safari():
        if SafariConfig.command_executor != "":
            return webdriver.Remote(command_executor=SafariConfig.command_executor,
                                    desired_capabilities=DesiredCapabilities.SAFARI.copy())
        driver = webdriver.Safari(executable_path='/usr/bin/safaridriver')
        driver.maximize_window()
        return driver


class WebDriver(object):
    def __init__(self, browser_name=None):
        self.d = Browser(browser_name)
        # 设置页面加载超时时间
        try:
            timeout = int(conf.get_item('common', 'timeout'))
        except:
            pass
        else:
            self.d.set_page_load_timeout(timeout)

    def open_url(self, url, login=True):
        if "http" not in url:
            base_url = conf.get_item('common', 'base_url')
            if 'http' in base_url:
                url = parse.urljoin(base_url, url)
            else:
                logger.debug('请设置正确的base_url')
                sys.exit()
        logger.info(f'访问: {url}')
        self.d.get(url)
        # 把默认请求头添加到cookie中
        if login:
            headers = conf.get_item('common', 'headers')
            logger.debug(headers)
            cookies = []
            if headers:
                for k, v in headers.items():
                    cookies.append({"name": k, "value": v})
                self.add_cookies(cookies)
                self.refresh()

    def back(self):
        logger.info('返回上一页')
        self.d.back()

    def screenshot(self, file_name: str, rect=None):
        """
        截图并保存到预定路径并上传到allure报告
        @param file_name: foo.png or fool
        @param rect: 列表
        @return:
        """
        # 把文件名处理成test.png的样式
        if '.' in file_name:
            file_name = file_name.split(r'.')[0]
        # 截图并保存到当前目录的images文件夹中
        img_dir = os.path.join(os.getcwd(), 'images')
        if os.path.exists(img_dir) is False:
            os.mkdir(img_dir)
        time_str = time.strftime('%Y年%m月%d日 %H时%M分%S秒')
        file_path = os.path.join(img_dir,
                                 f'[{time_str}]-{file_name}.png')
        logger.info(f'截图保存至: {file_path}')
        self.d.save_screenshot(file_path)

        # 标记
        if rect is not None:
            img = Image(file_path)
            img.rectangle(Rect(*rect), color=(0, 0, 255), thickness=3)
            img.imwrite(file_path)

        # 截图并上传到allure报告
        logger.info(f'截图上传至allure报告')
        allure.attach.file(file_path,
                           attachment_type=allure.attachment_type.PNG,
                           name=f'[{time_str}]-{file_name}')

    @property
    def page_content(self):
        page_source = self.d.page_source
        logger.info(f'获取页面内容: \n{page_source}')
        return page_source

    def max_window(self):
        logger.info('窗口设置全屏')
        self.d.maximize_window()

    def set_window(self, width, height):
        logger.info('设置窗口长宽')
        self.d.set_window_size(width, height)

    def get_windows(self):
        logger.info(f'获取当前打开的窗口列表')
        return self.d.window_handles

    def switch_window(self, old_windows):
        logger.info('切换到最新的window')
        current_windows = self.d.window_handles
        newest_window = [window for window in current_windows if window not in old_windows][0]
        self.d.switch_to.window(newest_window)

    def window_scroll(self, width, height):
        logger.info(f'设置scroll bar的宽高')
        js = "window.scrollTo({w},{h});".format(w=str(width), h=str(height))
        self.d.execute_script(js)

    def switch_to_frame(self, frame_id):
        logger.info(f'切换到frame {frame_id}')
        self.d.switch_to.frame(frame_id)

    def switch_to_frame_out(self):
        logger.info('从frame中切出来')
        self.d.switch_to.default_content()

    def execute_js(self, script, *args):
        logger.info(f'执行js脚本: \n{script}')
        self.d.execute_script(script, *args)

    def click(self, element):
        logger.info(f'点击元素: {element}')
        self.d.execute_script('arguments[0].click();', element)

    def quit(self):
        logger.info('退出浏览器')
        self.d.quit()

    def close(self):
        logger.info('关闭当前页签')
        self.d.close()

    def add_cookies(self, cookies: list):
        for cookie in cookies:
            self.d.add_cookie(cookie)

    def get_cookies(self):
        logger.info('获取cookies')
        cookies = self.d.get_cookies()
        logger.info(cookies)
        return cookies

    def get_cookie(self, name):
        logger.info(f'获取cookie: {name}')
        cookie = self.d.get_cookie(name)
        logger.info(cookie)
        return cookie

    def delete_all_cookies(self):
        logger.info('删除所有cookie')
        self.d.delete_all_cookies()

    def delete_cookie(self, name):
        logger.info(f'删除cookie: {name}')
        self.d.delete_cookie(name)

    def refresh(self):
        logger.info(f'刷新当前页')
        self.d.refresh()

    def get_title(self):
        logger.info('获取页面标题')
        title = self.d.title
        logger.info(title)
        return title

    def get_url(self):
        logger.info('获取页面url')
        url = self.d.current_url
        logger.info(url)
        return url

    def get_alert_text(self):
        logger.info('获取alert的文本')
        alert_text = self.d.switch_to.alert.text
        logger.info(alert_text)
        return alert_text

    def accept_alert(self):
        logger.info('同意确认框')
        self.d.switch_to.alert.accept()

    def dismiss_alert(self):
        logger.info('取消确认框')
        self.d.switch_to.alert.dismiss()





from .. import Provider as AddressProvider


class Provider(AddressProvider):
    city_formats = ("{{first_name}}",)
    city_suffixes = ("şəhəri", "ş.")
    street_suffixes = ("küçəsi", "küç.", "prospekti", "pr.")
    village_suffixes = ("kəndi", "k.")
    district_suffixes = ("rayonu", "ray.")
    settlement_suffixes = ("qəsəbəsi", "qəs.")

    building_number_formats = ("#",)
    house_number_formats = ("#", "##", "###")
    address_formats = (
        "{{city}} {{city_suffix }}, {{street_name}} {{street_suffix}}, b. {{building_number}}, m. {{house_number}}",
        "{{district}} {{district_suffix }}, {{street_name}} {{street_suffix}}, b. {{building_number}},"
        " m. {{house_number}}",
        "{{district}} {{district_suffix }}, {{village}} {{village_suffix}}, {{street_name}} {{street_suffix}}",
        "{{district}} {{district_suffix }}, {{settlement}} {{settlement_suffix}}, {{street_name}} {{street_suffix}},"
        " b. {{building_number}}, m. {{house_number}}",
    )
    street_name_formats = ("{{street}}",)
    cities = [
        "Bakı",
        "Gəncə",
        "Naxçıvan",
        "Xankəndi",
        "Lənkəran",
        "Mingəçevir",
        "Naftalan",
        "Sumqayıt",
        "Şəki",
        "Şirvan",
        "Yevlax",
    ]
    countries = [
        "Abxaziya",
        "Akrotiri və Dekeliya",
        "Aland adaları",
        "Albaniya",
        "Almaniya",
        "Amerika Samoası",
        "Andorra Knyazlığı",
        "Angilya",
        "Anqola",
        "Antiqua və Barbuda",
        "Argentina Respublikası",
        "Aruba",
        "Avstraliya",
        "Avstriya",
        "Azərbaycan",
        "Baham adaları",
        "Banqladeş",
        "Barbados",
        "Belçika",
        "Beliz",
        "Belarus",
        "Benin",
        "Bermud adaları",
        "Birləşmiş Ərəb Əmirlikləri",
        "ABŞ",
        "Boliviya",
        "Bolqarıstan",
        "Bosniya və Herseqovina",
        "Botsvana",
        "Böyük Britaniya",
        "Braziliya",
        "Bruney",
        "Burkina",
        "Burundi",
        "Butan",
        "Bəhreyn",
        "Cersi",
        "Cəbəllütariq",
        "Cənubi Afrika Respublikası",
        "Cənubi Sudan",
        "Cənubi Koreya",
        "Cibuti",
        "Çad",
        "Çexiya",
        "Monteneqro",
        "Çili",
        "Çin",
        "Danimarka",
        "Dominika",
        "Dominikan",
        "Efiopiya",
        "Ekvador",
        "Ekvatorial",
        "Eritreya",
        "Ermənistan",
        "Estoniya",
        "Əfqanıstan",
        "Əlcəzair",
        "Farer adaları",
        "Fələstin",
        "Fici",
        "Fil Dişi Sahili",
        "Filippin",
        "Finlandiya",
        "Folklend adaları",
        "Fransa",
        "Fransa Polineziyası",
        "Gernsi",
        "Gürcüstan",
        "Haiti",
        "Hindistan",
        "Honduras",
        "Honkonq",
        "Xorvatiya",
        "İndoneziya",
        "İordaniya",
        "İraq",
        "İran",
        "İrlandiya",
        "İslandiya",
        "İspaniya",
        "İsrail",
        "İsveç",
        "İsveçrə",
        "İtaliya",
        "Kabo-Verde",
        "Kamboca",
        "Kamerun",
        "Kanada",
        "Kayman adaları",
        "Keniya",
        "Kipr",
        "Kiribati",
        "Kokos adaları",
        "Kolumbiya",
        "Komor adaları",
        "Konqo Respublikası",
        "Konqo Demokratik Respublikası",
        "Kosovo",
        "Kosta-Rika",
        "Kuba",
        "Kuk adaları",
        "Küveyt",
        "Qabon",
        "Qambiya",
        "Qana",
        "Qətər",
        "Qayana",
        "Qazaxıstan",
        "Qərbi Saxara",
        "Qırğızıstan",
        "Qrenada",
        "Qrenlandiya",
        "Quam",
        "Qvatemala",
        "Qvineya",
        "Qvineya-Bisau",
        "Laos",
        "Latviya",
        "Lesoto",
        "Liberiya",
        "Litva",
        "Livan",
        "Liviya",
        "Lixtenşteyn",
        "Lüksemburq",
        "Macarıstan",
        "Madaqaskar",
        "Makao",
        "Şimali Makedoniya",
        "Malavi",
        "Malayziya",
        "Maldiv adaları",
        "Mali",
        "Malta",
        "Marşall adaları",
        "Mavriki",
        "Mavritaniya",
        "Mayotta",
        "Meksika",
        "Men adası",
        "Mərakeş",
        "Mərkəzi Afrika Respublikası",
        "Mikroneziya",
        "Milad adası",
        "Misir",
        "Myanma",
        "Moldova",
        "Monako",
        "Monqolustan",
        "Montserrat",
        "Mozambik",
        "Müqəddəs Yelena adası",
        "Namibiya",
        "Nauru",
        "Nepal",
        "Niderland",
        "Niger",
        "Nigeriya",
        "Nikaraqua",
        "Norveç",
        "Oman",
        "Özbəkistan",
        "Pakistan",
        "Palau",
        "Panama",
        "Papua",
        "Paraqvay",
        "Peru",
        "Pitkern adaları",
        "Polşa",
        "Portuqaliya",
        "Puerto-Riko",
        "Ruanda",
        "Rumıniya",
        "Rusiya",
        "Salvador",
        "Samoa",
        "San-Marino",
        "San-Tome və Prinsipi",
        "Seneqal",
        "Sen-Bartelemi",
        "Sent-Kits və Nevis",
        "Sent-Lüsiya",
        "Sen-Marten",
        "Sen-Pyer və Mikelon",
        "Sent-Vinsent və Qrenadin",
        "Serbiya",
        "Seyşel adaları",
        "Səudiyyə Ərəbistanı",
        "Sinqapur",
        "Slovakiya",
        "Sloveniya",
        "Solomon adaları",
        "Somali",
        "Somalilend",
        "Sudan",
        "Surinam",
        "Suriya",
        "Esvatini",
        "Syerra-Leone",
        "Şərqi Timor",
        "Şimali Kipr Türk Respublikası",
        "Şimali Koreya",
        "Şimali Marian adaları",
        "Şri-Lanka",
        "Tacikistan",
        "Tanzaniya",
        "Tailand",
        "Tonqa",
        "Toqo",
        "Trinidad və Tobaqo",
        "Tunis",
        "Tuvalu",
        "Türkiyə",
        "Türkmənistan",
        "Ukrayna",
        "Uqanda",
        "Uruqvay",
        "Vanuatu",
        "Vatikan",
        "Venesuela",
        "Vyetnam",
        "Yamayka",
        "Yaponiya",
        "Yeni Kaledoniya",
        "Yeni Zelandiya",
        "Yəmən",
        "Yunanıstan",
        "Zambiya",
        "Zimbabve",
    ]
    districts = [
        "Abşeron",
        "Ağcabədi",
        "Ağdam",
        "Ağdaş",
        "Ağstafa",
        "Ağsu",
        "Astara",
        "Babək",
        "Balakən",
        "Beyləqan",
        "Bərdə",
        "Biləsuvar",
        "Cəbrayıl",
        "Cəlilabad",
        "Culfa",
        "Daşkəsən",
        "Füzuli",
        "Gədəbəy",
        "Goranboy",
        "Göyçay",
        "Göygöl",
        "Hacıqabul",
        "Xaçmaz",
        "Xızı",
        "Xocalı",
        "Xocavənd",
        "İmişli",
        "İsmayıllı",
        "Kəlbəcər",
        "Kəngərli",
        "Kürdəmir",
        "Qax",
        "Qazax",
        "Qəbələ",
        "Qobustan",
        "Quba",
        "Qubadlı",
        "Qusar",
        "Laçın",
        "Lerik",
        "Lənkəran",
        "Masallı",
        "Neftçala",
        "Oğuz",
        "Ordubad*",
        "Saatlı",
        "Sabirabad",
        "Salyan",
        "Samux",
        "Sədərək",
        "Siyəzən",
        "Şabran",
        "Şahbuz",
        "Şamaxı",
        "Şəki",
        "Şəmkir",
        "Şərur",
        "Şuşa",
        "Tərtər",
        "Tovuz",
        "Ucar",
        "Yardımlı",
        "Yevlax",
        "Zaqatala",
        "Zəngilan",
        "Zərdab",
    ]
    settlements = [
        "Ləki",
        "Nehrəm",
        "Soyuqbulaq",
        "Şəkərli",
        "Cəhri",
        "Qarayazi",
        "Həzi asalanov",
        "Poylu",
        "Ceyrançöl",
        "Saloğlu",
        "Vurğun",
        "Qəhramanli",
        "Yuxari aran",
        "Mayak",
        "Milabad",
        "Örənqala",
        "Baharabad",
        "Günəş",
        "Orconikidze",
        "Digah",
        "Güzdək",
        "Hökməli",
        "Qobu",
        "Mehdiabad",
        "Saray",
        "Taxtakörpü",
        "Hindarx",
        "Şərq",
        "Sarisu",
        "Mil",
        "Türklər",
        "Bahar",
        "Babək",
        "Keşlə",
        "Qabaqçöl",
        "Xindiristan",
        "Bənövşələr",
        "Birinci alibəyli",
        "Birinci baharli",
        "Birinci dördyol",
        "Birinci quzanli",
        "İkinci alibəyli",
        "İkinci baharli",
        "İkinci dördyol",
        "İmamqulubəyli",
        "Qasimbəyli",
        "Səfərli",
        "Təzəkənd",
        "Ergi",
        "Yeni ergi",
        "Aşaği qəsil kəndi",
        "Orta qəsil kəndi",
        "Türyançay",
        "Yuxari qəsil kəndi",
        "Birinci zobucuq",
        "İkinci zobucuq",
        "Üçüncü zobucuq",
        "Dördüncü zobucuq",
        "Beşinci zobucuq",
        "Fin",
        "Horadiz",
        "Qayidiş 2",
        "Hacikənd",
        "Yuxari ağcakənd",
        "Qizilhacili",
        "Goran",
        "Aşaği ağcakənd",
        "Qazanbulaq",
        "Kürəkçay",
        "Ayaq qərvənd",
        "Quzanli",
        "Navahi",
        "Pirsaat",
        "Muğan",
        "Padar",
        "Navahi",
        "Vətəgə",
        "Bəhrəmtəpə",
        "Lahic",
        "Basqal",
        "Şəfəq",
        "Yeni mil",
        "Xocahəsən",
        "Sulutəpə",
        "Biləcəri",
        "Binəqədi",
        "28 may",
        "Rəsulzadə",
        "Qumlaq",
        "Xələfli",
        "Xudafərin",
        "Mahmudlu",
        "Novoqolovka",
        "Alunitdağ",
        "Quşçu körpüsü",
        "Daşkəsən",
        "Quşçu",
        "Yuxari daşkəsən",
        "1 nömrəli qayidiş",
        "2 nömrəli qayidiş",
    ]
    streets = [
        "A.AĞAYEV",
        "A.M.CÜMƏ",
        "TƏBRİZ",
        "XALİQ ABBASOV",
        "İSLAM ABIŞOV",
        "HEYDƏR ƏLİYEV",
        "XƏTAİ",
        "GÖL ƏTRAFI",
        "Z.XƏLİL",
        "2-Cİ SÜLH",
        "Q.PİRİMOV",
        "ASİF MƏMMƏDOV",
        "R.HACIYEV",
        "FƏXRƏDDİN ƏSƏDOV",
        "K.MARKS",
        "OKTAY KƏRİMOV",
        "Z.KAZIMZADƏ",
        "HƏSƏNOĞLU",
        "KAVEROÇKİN",
        "P.ÇAYKOVSKİ",
        "HÜSEYN ARİF",
        "HACI MURAD",
        "BAKI-BATUMİ",
        "NEMƏT QULİYEV",
        "R.AXUNDOV",
        "AKAD.H.ƏLİYEV",
        "RƏHİM ŞIXƏLİYEV",
        "YUSİFZADƏ",
        "E.QOCAYEV",
        "TARZƏN H.MƏMMƏDOV",
        "İ.HİDAYƏTZADƏ",
        "T.ƏLİYEV",
        "MƏMMƏD ARAZ",
        "V.PLOTNİKOV",
        "Ə.ORUCƏLİYEV",
        "Z.BÜNYADOV",
        "İ.DADAŞOV",
        "ƏLƏSGƏR QAYIBOV",
        "M.ARİF",
        "M.QASQAY",
        "Ə.ƏBDÜLRƏHİMOV",
        "İZZƏT HƏMİDOV",
        "AZADLIQ",
        "ARİF HEYDƏROV",
        "N.SÜLEYMANOV",
        "ŞAHİN MUSTAFAYEV",
        "Ə.VAHİD",
        "Ü.BÜNYADZADƏ",
        "NAZİM HACIYEV",
        "24-CÜ KORPÜ",
        "1-Cİ MƏDƏN",
        "Y.HÜSEYNOV",
        "22-Cİ DAĞLIQ",
        "SÜD FERMASI",
        "ÇAPAYEV",
        "E. NƏCƏFOV",
        "FAİQ RÜSTƏMOV",
        "28 MAY",
        "ZABİTLƏR EVİ",
        "S.S. AXUNDOV",
        "GƏNCLƏR DOSTLUĞU",
        "H.SULTANOV",
        "ƏHMƏD QASIMOV",
        "XURŞUD AĞAYEV",
        "NATƏVAN",
        "YENİ MASSİV",
        "MƏLİK ASLANOV KÜÇƏSİ,",
        "VİDADİ",
        "8 MART",
        "İ.HACIYEV",
        "Y. HƏSƏNOV",
    ]
    villages = [
        "Kələki",
        "Binələr",
        "Davudlu",
        "Birinci aral",
        "İkinci aral",
        "Cardam",
        "Qaradeyin",
        "Qarağan şixlar",
        "Qarağan sədi",
        "Qəribli",
        "Qolqəti",
        "Mürsəl",
        "Şordəhnə",
        "Tofiqi",
        "Yenicə",
        "Ərəbocaği",
        "Hapitli",
        "Ovçulu",
        "Şəkili",
        "Yuxari ağcayazi",
        "Aşaği ağcayazi",
        "Yuxari ləki",
        "Düzqişlaq",
        "Kolayir",
        "Koçvəlili",
        "Xətai",
        "Yenigün",
        "Qaçaq kərəm",
        "Poylu",
        "Tatli",
        "Yaradullu",
        "Xilxina",
        "Mollacəfərli",
        "Ağgöl",
        "Aşaği göycəli",
        "Aşaği kəsəmən",
        "Böyük kəsik",
        "Dağ kəsəmən",
        "Eynalli",
        "Göycəli",
        "Həsənsu",
        "Kolxələfli",
        "Koçəsgər",
        "Köhnəqişlaq",
        "Qarahəsənli",
        "Soyuqbulaqlar",
        "Qiraq kəsəmən",
        "Didivar",
        "Muğanli",
        "Pirili",
        "Sadiqli",
        "Uzunoba",
        "Qaraməmmədli",
        "Navahi",
        "Ülgüc",
        "Ərəbmehdibəy",
        "Dədəli",
        "Qasimbəyli",
        "Ərəbsarvan",
        "Haciqədirli",
        "Göydəlləkli",
        "Ərəbuşaği",
        "Ağalarbəyli",
        "Maşadqanli",
        "Aratli curuğlu",
        "Keşdiməz",
        "Bozavand",
        "Ağarx",
        "Qarabağli",
        "Xanbulaq",
        "Kəndaxan",
        "Yenilik",
        "Kövlüc",
        "Elabad",
        "Yenikənd",
        "Hingar",
        "Girdə",
        "Gursulu",
        "Qaraqoyunlu",
        "Musabəyli",
        "İlxiçi",
        "Hacisəmədli",
        "Qəşəd",
        "Kəndoba",
        "Cəfərli",
        "Haciuşaği",
        "Cəlayir",
        "Abasxanli",
        "Kalva",
        "Suraxani",
        "Dilman",
        "Haciman",
        "Xatman",
        "Növcü",
        "Axundlu",
        "Məlikçobanli",
    ]

    def house_number(self):
        """
        :example: 'm. 49'
        """
        return self.numerify(self.random_element(self.house_number_formats))

    def city(self):
        """
        :example: 'Xankəndi'
        """
        return self.random_element(self.cities)

    def city_suffix(self):
        """
        :example: 'ş.'
        """
        return self.random_element(self.city_suffixes)

    def street(self):
        """
        :example: 'A.AĞAYEV'
        """
        return self.random_element(self.streets)

    def street_suffix(self):
        """
        :example: 'küç.'
        """
        return self.random_element(self.street_suffixes)

    def village(self):
        """
        :example: 'Didivar'
        """
        return self.random_element(self.villages)

    def village_suffix(self):
        """
        :example: 'k.'
        """
        return self.random_element(self.village_suffixes)

    def district(self):
        """
        :example: 'Babək'
        """
        return self.random_element(self.districts)

    def district_suffix(self):
        """
        :example: 'r.'
        """
        return self.random_element(self.district_suffixes)

    def settlement(self):
        """
        :example: 'Horadiz'
        """
        return self.random_element(self.settlements)

    def settlement_suffix(self):
        """
        :example: 'qəs.'
        """
        return self.random_element(self.settlement_suffixes)

    def administrative_unit(self):
        """
        :example: 'Xankəndi'
        """
        return self.random_element(self.districts + self.cities)

    def postcode(self):
        """
        :example: 'AZ1027'
        """
        index = self.generator.random.randint(900, 6600)
        return "AZ%04d" % index if index > 999 else "AZ0%03d" % index

    def postalcode(self):
        return self.postcode()



# coding=utf-8
from qrunner.utils.faker.sphinx.docstring import ProviderMethodDocstring
from qrunner.utils.faker.sphinx.documentor import write_provider_docs


def _create_source_files(app):
    write_provider_docs()


def _process_docstring(app, what, name, obj, options, lines):
    docstring = ProviderMethodDocstring(app, what, name, obj, options, lines)
    if not docstring.skipped:
        lines[:] = docstring.lines[:]


def setup(app):
    app.setup_extension("sphinx.ext.autodoc")
    app.connect("builder-inited", _create_source_files)
    app.connect("autodoc-process-docstring", _process_docstring)



# @Time    : 2022/2/11 9:00
# @Author  : kang.yang@qizhidao.com
# @File    : swagger.py
import sys
import requests
import urllib3
urllib3.disable_warnings()


def is_chinese(string):
    """
    检查整个字符串是否包含中文
    :param string: 需要检查的字符串
    :return: bool
    """
    for ch in string:
        if u'\u4e00' <= ch <= u'\u9fff':
            return True

    return False


def get_swagger_data(swagger_url):
    """
    通过swagger接口获取接口列表
    @param swagger_url:
    @return: [
        ['请求方法', '项目名', '模块名', '模块描述', '接口名', '接口描述'],
        ...
    ]
    """
    # 请求url，获取返回的json
    res = requests.get(swagger_url, verify=False)
    # print(res.text)
    data_json: dict = res.json()
    # print(data_json)
    # 获取接口所属模块
    project: str = data_json.get('basePath')
    project = project.split('/')[1]
    # 获取tag名称和描述的映射关系
    tags = data_json.get('tags')
    tag_dict = {}
    for tag in tags:
        name = tag.get('name')
        des = tag.get('description')
        if name not in tag_dict:
            tag_dict[name] = des
    print(tag_dict)
    # 获取接口信息
    paths = data_json.get('paths')
    api_list = []
    for apiPath, value in paths.items():
        apiPath = f'/{project}{apiPath}'
        apiPath = apiPath.replace('{', '')
        apiPath = apiPath.replace('}', '')
        for method, content in value.items():
            tag = content['tags'][0]
            if is_chinese(tag):
                moduleDesc = tag
                moduleName = tag_dict[tag]
            else:
                moduleName = tag
                moduleDesc = tag_dict[tag]
            moduleDesc = moduleDesc.replace("'", "")
            moduleDesc = moduleDesc.replace('"', '')

            apiDesc: str = content['summary']
            apiDesc = apiDesc.replace("'", "")
            apiDesc = apiDesc.replace('"', '')
            api_list.append([method, project, moduleName, moduleDesc, apiPath, apiDesc])
    for index, api in enumerate(api_list):
        print(index, api)
    return api_list












